/**
 * @file  AEBuffer.cpp
 * @brief Implementierung der Klasse CAEBuffer.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "AEBuffer.h"
#include "AEErrorDef.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CAEBuffer::CAEBuffer(short channelID)
{
	// Kanalnummer zuweisen
	m_channelID = channelID;

	// Member Initialisieren
	StartDatensatzB = 0;
	EndDatensatzB = 0;
	MaxDatB = 0;
	MaxListB = 0;
	
	// Zeiger auf Listenspeicher initialisieren
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		m_ucListRecordSize[i] = 0;
		m_listSize[i] = 0;
		m_listSortCheck[i] = false;
		m_pucListMem[i] = NULL;
	}


	// Datensatzgroesse fuer Zutrittslisten (Version 1) initialisieren.
	m_ucEntranceRecordSize[0] = ENTRANCE_RECORD_SIZE_0; // 0: Zeitzonen (22)
	m_ucEntranceRecordSize[1] = ENTRANCE_RECORD_SIZE_1; // 1: Zeitmodelle (35)
	m_ucEntranceRecordSize[2] = ENTRANCE_RECORD_SIZE_2; // 2: Personalstamm (19)
	m_ucEntranceRecordSize[3] = ENTRANCE_RECORD_SIZE_3; // 3: Feiertage (16)

	// Zeiger auf Zutrittslistenspeicher (Version 1) initialisieren
	for (i=0; i < 4; i++) {

		m_ulEntranceListSize[i] = 0;
		m_pucEntranceListMem[i] = NULL;
	}


	// Datensatzgroesse fuer Zutrittslisten (Version 2) initialisieren.
	m_ucEntrance2RecordSize[0] = ENTRANCE2_RECORD_SIZE_0;
	m_ucEntrance2RecordSize[1] = ENTRANCE2_RECORD_SIZE_1;
	m_ucEntrance2RecordSize[2] = ENTRANCE2_RECORD_SIZE_2;
	m_ucEntrance2RecordSize[3] = ENTRANCE2_RECORD_SIZE_3;
	m_ucEntrance2RecordSize[4] = ENTRANCE2_RECORD_SIZE_4;
	m_ucEntrance2RecordSize[5] = ENTRANCE2_RECORD_SIZE_5;
	m_ucEntrance2RecordSize[6] = ENTRANCE2_RECORD_SIZE_6;

	// Zeiger auf Zutrittslistenspeicher (Version 2) initialisieren
	for (i=0; i < 7; i++) {

		m_ulEntrance2ListSize[i] = 0;
		m_entrance2SortCheck[i] = false;
		m_pucEntrance2ListMem[i] = NULL;
	}

	// Zutrittskontrollstrukturen (Version 2) aufsetzen.
	Entrance2Default();

	// Zeiger auf Timeboylistenspeicher initialisieren
	memset(m_pucTimeboyListMem, 0, sizeof(m_pucTimeboyListMem));
	ClearTimeboyListBuffer();

	// Seriennummer der Geraetedatei auf 0 setzen
	m_fileDevice.SetSerial(0);

	// Speicher des Dokuments initialisieren
	SetDocumentDefault();
}

CAEBuffer::~CAEBuffer()
{
	// Reservierten Listenspeicher freigeben
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		if (m_pucListMem[i]) {

			delete[] m_pucListMem[i];
			m_pucListMem[i] = NULL;
		}
	}

	// Reservierten Zutrittslistenspeicher (Version 1) freigeben
	for (i=0; i < 4; i++) {

		if (m_pucEntranceListMem[i]) {

			delete[] m_pucEntranceListMem[i];
			m_pucEntranceListMem[i] = NULL;
		}
	}

	// Reservierten Zutrittslistenspeicher (Version 2) freigeben
	for (i=0; i < 7; i++) {

		if (m_pucEntrance2ListMem[i]) {

			delete[] m_pucEntrance2ListMem[i];
			m_pucEntrance2ListMem[i] = NULL;
		}
	}

	// Reservierten Timeboylistenspeicher freigeben
	ClearTimeboyListBuffer();
}

//////////////////////////////////////////////////////////////////////
// Member
//////////////////////////////////////////////////////////////////////

void CAEBuffer::SetDocumentDefault()
{
	// Eeprom 
	memset(m_pucEepMem, 0xFF, sizeof(m_pucEepMem));
	
	// Flashbereiche
	memset(m_pucFlashMem, 0xFF, sizeof(m_pucFlashMem));

	// Konfigbereich
	memset(m_pucConfigMem, 0, sizeof(m_pucConfigMem));

	// Extended Flash
	memset(m_pucExtendedMem, 0, sizeof(m_pucExtendedMem));

	// Der erweiterte Speicherbereich wird mit einer dynamischen Struktur begonnen,
	// deren erster Parameter ein Check - UINT32 - Wert enthaelt.
	TExtendedFlash extBegin;
	// Alles mit Null initialisieren.
	memset(&extBegin, 0, sizeof(TExtendedFlash));
	// Struktur fuer erweiterten Bereich aufsetzen
	extBegin.ulCheck = EXTENDED_CHECK;
	extBegin.ulSize  = sizeof(TExtendedFlash);
	extBegin.pulFreeScope = extBegin.ulSize;
	// Flag fuer Verarbeitung
	extBegin.ulFlags = EXTF_NEW;
	
	*(TExtendedFlash*)m_pucExtendedMem = extBegin;
}

void CAEBuffer::ChangeOrderLong(unsigned char *Buf)
{
	unsigned char h;

	h = Buf[0];
	Buf[0] = Buf[3];
	Buf[3] = h;

	h = Buf[1];
	Buf[1] = Buf[2];
	Buf[2] = h;
}

void CAEBuffer::ULongToBinMSBFirst(unsigned char *Buf,unsigned long Val)
{
	Buf[3] = (unsigned char)(Val&0xFF);
	Val >>= 8;
	Buf[2] = (unsigned char)(Val&0xFF);
	Val >>= 8;
	Buf[1] = (unsigned char)(Val&0xFF);
	Val >>= 8;
	Buf[0] = (unsigned char)(Val&0xFF);
}

void CAEBuffer::ChangeOrderInt(unsigned char *Buf)
{
	unsigned char h;
	h = Buf[0];
	Buf[0] = Buf[1];
	Buf[1] = h;
}

void CAEBuffer::ClearListBuffer()
{
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		m_listSize[i] = 0;
		if (m_pucListMem[i]) {

			delete[] m_pucListMem[i];
			m_pucListMem[i] = NULL;
		}
	}
}

int CAEBuffer::MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
	int i;

	// Zuvor reservierten Speicher freigeben
	if (m_pucListMem[ListNum]) {

		delete[] m_pucListMem[ListNum];
		m_pucListMem[ListNum] = NULL;
	}

	// Speicher reservieren
	m_pucListMem[ListNum] = new unsigned char[ListSize + sizeof(unsigned long)];
	
	if (m_pucListMem[ListNum] == NULL) {

		return FALSE;
	}

	// Pointer in Tabelle der Listen-Pointer eintragen
//	*(unsigned long*)&m_pucListMem[ListNum * sizeof(unsigned long)] = m_ulListSize + m_ulListStart;
//	ChangeOrderLong((unsigned char*)&m_pucListMem[ListNum * sizeof(unsigned long)]);

	// Start fuer Listendaten berechnen
//	pDat = m_ulListSize + sizeof(unsigned long);	// Platz fuer Anzahl Zeilen freihalten

	// Datenueberlauf auffangen
/*	if ((pDat + ListSize) > sizeof(m_pucListMem)) {
		// Liste loeschen
		*(unsigned long*)&m_pucListMem[ListNum * sizeof(unsigned long)] = 0;
		return FALSE;
	}
*/	
	// Listendaten kopieren
	for (i=0; i < ListSize; i++) {

		m_pucListMem[ListNum][sizeof(unsigned long) + i] = Buf[i];
	}

	// Anzahl Listenzeilen eintragen
	*(unsigned long*)&m_pucListMem[ListNum][0] = ListCnt;
	//ChangeOrderLong(&m_pucListMem[ListNum][0]);
	
	// Listen-Pointer auf erstes freies Byte im Listenspeicher setzen
	//m_ulListSize = pDat + ListSize;

	// Liste als unsortiert markieren
	m_listSortCheck[ListNum] = false;

	// Listengrosse fuer weitere Verarbeitung halten
	m_listSize[ListNum] = sizeof(unsigned long) + ListSize;

	return TRUE;
}

void CAEBuffer::QuicksortListRecord(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum)
{
	long lHelp1, lHelp2;

	if (lRightRecordNum > lLeftRecordNum) {

		//CString strTmp((const unsigned char*)&m_pucListMem[ucNum][lRightRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]);
		lHelp1 = lLeftRecordNum - 1;
		lHelp2 = lRightRecordNum;

		for (;;) {
		
			//while (CString((const unsigned char*)&m_pucListMem[ucNum][++lHelp1 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) < strTmp);
			//while (CString((const unsigned char*)&m_pucListMem[ucNum][--lHelp2 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) > strTmp);

			while (strcmp((const char*)&m_pucListMem[ucNum][++lHelp1 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				          (const char*)&m_pucListMem[ucNum][lRightRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) < 0);
			while (strcmp((const char*)&m_pucListMem[ucNum][--lHelp2 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				          (const char*)&m_pucListMem[ucNum][lRightRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) > 0);

			if (lHelp1 >= lHelp2) {
			
				break;
			}

			if (strcmp((const char*)&m_pucListMem[ucNum][lHelp1 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				       (const char*)&m_pucListMem[ucNum][lHelp2 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) != 0) {
			
				SwapListRecord(ucNum, lHelp1, lHelp2);
			}
		}

		if (strcmp((const char*)&m_pucListMem[ucNum][lHelp1 * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				   (const char*)&m_pucListMem[ucNum][lRightRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) != 0) {
		
			SwapListRecord(ucNum, lHelp1, lRightRecordNum);
		}

		QuicksortListRecord(ucNum, ucOffs, lLeftRecordNum, lHelp1 - 1);
		QuicksortListRecord(ucNum, ucOffs, lHelp1 + 1, lRightRecordNum);
	}
}

void CAEBuffer::SwapListRecord(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum)
{
	int i;
	unsigned char ucTmp;
	for (i=0; i < m_ucListRecordSize[ucNum]; i++) {
	
		ucTmp = m_pucListMem[ucNum][lLeftRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + i];
		m_pucListMem[ucNum][lLeftRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + i] = m_pucListMem[ucNum][lRightRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + i];
		m_pucListMem[ucNum][lRightRecordNum * m_ucListRecordSize[ucNum] + sizeof(unsigned long) + i] = ucTmp;
	}
}


void CAEBuffer::ClearEntranceListBuffer(int iListNum)
{
	int i, j;
	if (iListNum == -1) { // Alle loeschen

		for (i=0; i < 4; i++) {
		
			m_ulEntranceListSize[i] = 0;
			m_ulCompressedListSize[i] = 0;
		}
	
		for (i=0; i < 4; i++) {

			// Reservierten Speicher freigeben
			if (m_pucEntranceListMem[i]) {

				delete[] m_pucEntranceListMem[i];
				m_pucEntranceListMem[i] = NULL;
			}

			for(j=0; j < FLASH_SIZE_LIST; j++) {

				m_pucCompressedListMem[i][j] = 0;
			}
		}

	} else {

		if (iListNum > 3) return;
		m_ulEntranceListSize[iListNum] = 0;


		// Reservierten Speicher freigeben
		if (m_pucEntranceListMem[iListNum]) {

			delete[] m_pucEntranceListMem[iListNum];
			m_pucEntranceListMem[iListNum] = NULL;
		}

		for (j=0; j < FLASH_SIZE_LIST; j++) {
		
			m_pucCompressedListMem[iListNum][j] = 0;
		}

		m_ulCompressedListSize[iListNum] = 0;
	}

}

int CAEBuffer::MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
	int i;

	// ListNum ist:
	// 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
	// 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
	// 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
	// 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich
	
	// Zuvor reservierten Speicher freigeben
	if (m_pucEntranceListMem[ListNum]) {

		delete[] m_pucEntranceListMem[ListNum];
		m_pucEntranceListMem[ListNum] = NULL;
	}

	// Speicher reservieren
	m_pucEntranceListMem[ListNum] = new unsigned char[ListSize + sizeof(unsigned long)];

	if (m_pucEntranceListMem[ListNum] == NULL) {

		return FALSE;
	}

	// Rohe Listendaten kopieren
	for (i=0; i < ListSize; i++) {

		m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i] = Buf[i]; // Platz fuer Anzahl Zeilen freihalten
	}


	// Anzahl Listenzeilen eintragen
	*(unsigned long*)&m_pucEntranceListMem[ListNum][0] = ListCnt;
	ChangeOrderLong(&(m_pucEntranceListMem[ListNum][0]));
	
	// Listenspeichergroesse setzen
	m_ulEntranceListSize[ListNum] = sizeof(unsigned long) + ListSize;


	// ##########################################
	// Komprimieren und sortieren der Listendaten
	// ##########################################

	// Liste Zeitzone - Komprimieren
	if (ListNum == 0) {

		structZeitZone *pZeitZone;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_0;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_0];

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(unsigned long);

		// Liste mit -zeitzone- Daten komprimieren
		for (i=0; i < (unsigned long)ListCnt; i++) {
			
			for (j=0; j < ucRecordSize; j++) {

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}
			}

			pZeitZone = (structZeitZone*)&m_pucCompressedListMem[ListNum][sizeof(unsigned long) + i * sizeof(structZeitZone)];
			pZeitZone->day = (unsigned char)((1 << pucRecord[0]) | (1 << pucRecord[1]) | (1 << pucRecord[2]) | (1 << pucRecord[3]) | (1 << pucRecord[4]) | (1 << pucRecord[5]) | (1 << pucRecord[6]));
			pZeitZone->start = ((unsigned short)(pucRecord[8] * 10 + pucRecord[9]) << 8) | (pucRecord[11] * 10 + pucRecord[12]);
			ChangeOrderInt((unsigned char*)&pZeitZone->start);
			pZeitZone->ende = ((unsigned short)(pucRecord[14] * 10 + pucRecord[15]) << 8) | (pucRecord[17] * 10 + pucRecord[18]);
			ChangeOrderInt((unsigned char*)&pZeitZone->ende);
			pZeitZone->aktiv = pucRecord[20];

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structZeitZone);
		}
	}

	// Liste Zeitmodelle - Komprimieren
	if (ListNum == 1) {

		structZeitModell *pZeitModell;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_1;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_1];
		unsigned char ucOffs = 0;

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(unsigned long);

		// Liste mit -zeitzmodelle- Daten komprimieren
		for (i=0; i < (unsigned long)ListCnt; i++) {
			
			for (j=0; j < ucRecordSize; j++) {

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}
			}

			pZeitModell = (structZeitModell*)&m_pucCompressedListMem[ListNum][sizeof(unsigned long) + i * sizeof(structZeitModell)];
			pZeitModell->zeitmodell = pucRecord[0] * 10 + pucRecord[1];
			ucOffs = 3;
			pZeitModell->zone = 0;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0001 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0002 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0004 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0008 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0010 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0020 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0040 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0080 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0100 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0200 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0400 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0800 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x1000 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x2000 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x4000 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x8000 : 0x0000;

			ChangeOrderInt((unsigned char*)&pZeitModell->zone);

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structZeitModell);
		}
	}

	// Liste Personalstamm - Komprimieren
	if (ListNum == 2) {
		
		// Personalstamm nach Personalnummer sortieren.
		QuicksortEntranceRecord(ListNum, 0, 0, ListCnt - 1);


		structPersonalStamm *pPersonalStamm;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_2;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_2];

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(unsigned long);
/*
		for (i=0; i < ListCnt; i++) {
		
			TRACE("%s\n", &m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize]);
		}
*/
		// Liste mit -personalstamm- Daten komprimieren
		for (i=0; i < (unsigned long)ListCnt; i++) {
			
			pPersonalStamm = (structPersonalStamm*)&m_pucCompressedListMem[ListNum][sizeof(unsigned long) + i * sizeof(structPersonalStamm)];
			
			for (j=0; j < ucRecordSize; j++) {

				if (j < 13) {

					pPersonalStamm->pNr[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j];
				} 

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}				
			}


			for (j=0; j < 13; j++) {
				
				if (!pPersonalStamm->pNr[j]) { // 0terminie suchen
				
					break;
				}
			}
			for (; j < 14; j++) { // rest mit 0 belegen
				
				pPersonalStamm->pNr[j] = 0x00;
			}

			pPersonalStamm->zeitmodell = pucRecord[14] * 10 + pucRecord[15];			
			pPersonalStamm->prio = pucRecord[17];

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structPersonalStamm);
		}
	}
	
	// Liste Feiertage - Komprimieren
	if (ListNum == 3) {

		structFeiertag *pFeiertag;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_3;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_3];

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(unsigned long);

		// Liste mit -feiertage- Daten komprimieren
		for (i=0; i < (unsigned long)ListCnt; i++) {
			
			for (j=0; j < ucRecordSize; j++) {

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned long) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}
			}

			pFeiertag = (structFeiertag*)&m_pucCompressedListMem[ListNum][sizeof(unsigned long) + i * sizeof(structFeiertag)];
			pFeiertag->zeitmodell = pucRecord[0] * 10 + pucRecord[1];
			// JJJJMMDD
			pFeiertag->datum = (unsigned long)(((pucRecord[9] * 1000 + pucRecord[10] * 100 + pucRecord[11] * 10 + pucRecord[12]) << 16) | ((pucRecord[6] * 10 + pucRecord[7]) << 8) | (pucRecord[3] * 10 + pucRecord[4]));
			ChangeOrderLong((unsigned char*)&pFeiertag->datum);
			pFeiertag->prio = pucRecord[14];

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structFeiertag);
		}
	}
	
	return TRUE;
}

void CAEBuffer::QuicksortEntranceRecord(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum)
{
	long lHelp1, lHelp2;

	if (lRightRecordNum > lLeftRecordNum) {

		//CString strTmp((const unsigned char*)&m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]);
		lHelp1 = lLeftRecordNum - 1;
		lHelp2 = lRightRecordNum;

		for (;;) {
		
			//while (CString((const unsigned char*)&m_pucEntranceListMem[ucNum][++lHelp1 * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) < strTmp);
			//while (CString((const unsigned char*)&m_pucEntranceListMem[ucNum][--lHelp2 * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) > strTmp);

			while (strcmp((const char*)&m_pucEntranceListMem[ucNum][++lHelp1 * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				          (const char*)&m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) < 0);
			while (strcmp((const char*)&m_pucEntranceListMem[ucNum][--lHelp2 * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				          (const char*)&m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) > 0);

			if (lHelp1 >= lHelp2) {
			
				break;
			}

			SwapEntranceRecord(ucNum, lHelp1, lHelp2);
		}

		SwapEntranceRecord(ucNum, lHelp1, lRightRecordNum);
		QuicksortEntranceRecord(ucNum, ucOffs, lLeftRecordNum, lHelp1 - 1);
		QuicksortEntranceRecord(ucNum, ucOffs, lHelp1 + 1, lRightRecordNum);
	}
}

void CAEBuffer::SwapEntranceRecord(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum)
{
	int i;
	unsigned char ucTmp;
	for (i=0; i < m_ucEntranceRecordSize[ucNum]; i++) {
	
		ucTmp = m_pucEntranceListMem[ucNum][lLeftRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + i];
		m_pucEntranceListMem[ucNum][lLeftRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + i] = m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + i];
		m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + i] = ucTmp;
	}
}


void CAEBuffer::ClearEntrance2ListBuffer(int iListNum)
{
	int i;
	if (iListNum == -1) { // Alle loeschen

		for (i=0; i < 7; i++) {

			m_ulEntrance2ListSize[i] = 0;

			// Reservierten Speicher freigeben
			if (m_pucEntrance2ListMem[i]) {

				delete[] m_pucEntrance2ListMem[i];
				m_pucEntrance2ListMem[i] = NULL;
			}
		}

	} else {

		if (iListNum > 6) return;
		m_ulEntrance2ListSize[iListNum] = 0;

		// Reservierten Speicher freigeben
		if (m_pucEntrance2ListMem[iListNum]) {

			delete[] m_pucEntrance2ListMem[iListNum];
			m_pucEntrance2ListMem[iListNum] = NULL;
		}
	}
}

int CAEBuffer::MakeEntrance2List(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
	int iError = IV_ERROR_SUCCESS;

	// Liste in DLL - Verarbeitungsbuffer schreiben.
	if ((iError = CheckEntrance2List(ListNum, ListCnt, Buf, ListSize)) != IV_ERROR_SUCCESS) {
		
		if (iError < 0) {

			// Kommunikationsfehler
			iError = -iError;
		} else if (iError > 0) {

			// Funktionsfehler
			iError = IV_ERROR_PROCESSING;
		}

		return FALSE;
	}

	// Liste in DLL - Verarbeitungsbuffer schreiben.
	if ((iError = ImportEntrance2List(ListNum, ListCnt, Buf, ListSize)) != IV_ERROR_SUCCESS) {
		
		if (iError < 0) {

			// Kommunikationsfehler
			iError = -iError;
		} else if (iError > 0) {

			// Funktionsfehler
			iError = IV_ERROR_PROCESSING;
		}

		return FALSE;
	}

	// Neu importierte Listen als unsortiert markieren
	m_entrance2SortCheck[ListNum] = false;

	return TRUE;
}

int CAEBuffer::CheckEntrance2Dec(char *buf, int min, int max)
{
	int i; // Alles Ziffern?
	for (i=0; i < (int)strlen(buf); i++) {

		if (!isdigit(buf[i])) {
		
			return FALSE;
		}
	}
	// Bereich eingehalten?
	if (atoi(buf) >= min && atoi(buf) <= max) {

		return TRUE;
	}

	return FALSE;
}

int CAEBuffer::CheckEntrance2TM(char *buf)
{
	char tm[4], stich;
	// Format OK?
	if (strlen(buf) != 3) {
		return FALSE;
	}
	strcpy(tm, buf);
	stich = atoi(&buf[2]);
	if (stich > 1) {
		return FALSE;
	}

	// Bereich eingehalten?
	if (atoi(tm) >= 0 && atoi(tm) <= 31) {

		return TRUE;
	} else if (atoi(tm) == 32 && !stich) {

		return TRUE;
	}

	return FALSE;
}

int CAEBuffer::CheckEntrance2Date(char *buf)
{
	// Format OK?
	char date[11];
	int year, month, day;
	if (strlen(buf) != 10) {
		return FALSE;
	}
	if (buf[4] != '-' || buf[7] != '-') {
		return FALSE;
	}
	strcpy(date, buf);
	date[4] = 0; date[7] = 0;
	year = atoi(date);
	month = atoi(&date[5]);
	day = atoi(&date[8]);
	if (month < 1 || month > 12) {
		return FALSE;
	}
	if (day < 1 || day > 31) {
		return FALSE;
	}

	return TRUE;
}

int CAEBuffer::CheckEntrance2Time(char *buf)
{
	// Format OK?
	char time[5];
	int hour, minute;
	if (strlen(buf) != 5) {
		return FALSE;
	}
	if (buf[2] != ':') {
		return FALSE;
	}
	strcpy(time, buf);
	time[2] = 0;
	hour = atoi(time);
	minute = atoi(&time[3]);
	if (hour < 0 || hour > 23) {
		return FALSE;
	}
	if (minute < 0 || minute > 59) {
		return FALSE;
	}

	return TRUE;
}

int CAEBuffer::CheckEntrance2Weekday(char *buf)
{
	// Format OK?
	unsigned char wd = 0;
	int i, val;
	if (strlen(buf) < 1 || strlen(buf) > 7) {
		return FALSE;
	}
	for (i=0; i < (int)strlen(buf); i++) {
		val = buf[i] - 0x30;
		if (val < 1 || val > 7) {
			return FALSE;
		}
		if (wd & (1 << val)) {
			return FALSE;
		}

		wd |= (1 << val);
	}

	return TRUE;
}

int CAEBuffer::CheckEntrance2List(unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned long size)
{
	// ListNum ist:
	// 0: Reader         (33 Byte)
	// 1: Identification (64 Byte)
	// 2: Location       (20 Byte)
	// 3: Time           (25 Byte)
	// 4: Holiday        (21 Byte)
	// 5: Events         (17 Byte)
	// 6: Action         (21 Byte)
	int checkRecords = FALSE;
	unsigned short recordNum;
	unsigned char fieldNum, recordPos = 0;

	// Bereich der gueltigen Nummer pruefen.
	switch(num) {
	case 0:
		// Anzahl moeglicher Datensaetze
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_0))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Berechnete Listengroesze pruefen
		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_0)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // ID
					case 1: // ZM
					case 3: // RefLocation
					case 4: // RefAction
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					case 2: // TM
						if (!CheckEntrance2TM((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos])) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					case 5: // PinGeneral
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 0, 99999999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;
	case 1:
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_1))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_1)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // ID
						break;
					case 1: // Group
					case 3: // Menace
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					case 2: // Pin
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 0, 99999999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					case 4: // ActiveStart
					case 5: // ActiveEnd
						if (!CheckEntrance2Date((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos])) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					case 6: // ActiveGeneral
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 0, 1)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;
	case 2:
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_2))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_2)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // ID
					case 1: // RefGroup
					case 2: // RefTime
					case 3: // RefTimeNoPin
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;
	case 3:
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_3))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_3)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // ID
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					case 1: // Weekday
						if (!CheckEntrance2Weekday((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos])) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					case 2: // TimeStart
					case 3: // TimeEnd
						if (!CheckEntrance2Time((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos])) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;
	case 4:
		// Anzahl moeglicher Datensaetze
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_4))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}
		// Berechnete Listengroesze pruefen
		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_4)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // Day
						if (!CheckEntrance2Date((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos])) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
						break;
					case 1: // RefGroup
					case 2: // RefTime
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;
	case 5:
		// Anzahl moeglicher Datensaetze
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_5))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}
		// Berechnete Listengroesze pruefen
		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_5)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // RefReader
					case 2: // RefAction
					case 3: // RefTime
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					case 1: // PortIn
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 3)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;
	case 6:
		// Anzahl moeglicher Datensaetze
		if ((recordCount < 1) /*|| (recordCount > (FLASH_SIZE_LIST / ENTRANCE_EXT_RECORD_SIZE_6))*/) {

			return -IV_ERROR_FUNCTION_PARAMETER;
		}
		// Berechnete Listengroesze pruefen
		if (size != (unsigned long)(recordCount * ENTRANCE2_RECORD_SIZE_6)) {
		
			return -IV_ERROR_FUNCTION_PARAMETER;
		}

		// Ueberpruefen der Listendaten durchfuehren?
		if (checkRecords) {

			// Datensaetze pruefen
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				// Spalten durchlaufen
				for (fieldNum=0; fieldNum < m_entrance2Table[num].count; fieldNum++) {
				
					switch(fieldNum) {
					case 0: // ID
					case 1: // RefReader
					case 4: // RefTime
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 9999)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					case 2: // PortOut
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 1, 2)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					case 3: // Elapse
						if (!CheckEntrance2Dec((char*)&buf[recordNum * ENTRANCE2_RECORD_SIZE_0 + recordPos], 0, 248)) {
							TRACE("%s:(%i):WARNING(1): CAEBuffer::CheckEntrance2List [  ]\n", __FILE__, __LINE__);
						}
					}

					recordPos += m_entrance2TableField[num][fieldNum].size;
				}
			}
		}
		break;

	default:
		return -IV_ERROR_FUNCTION_PARAMETER;
	}

	return IV_ERROR_SUCCESS;
}

int CAEBuffer::ImportEntrance2List(unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned long size)
{
	unsigned long i;

	// ListNum ist:
	// 0: Reader         (33 Byte)
	// 1: Identification (64 Byte)
	// 2: Location       (20 Byte)
	// 3: Time           (25 Byte)
	// 4: Holiday        (21 Byte)
	// 5: Events         (17 Byte)
	// 6: Action         (21 Byte)

	// Zuvor reservierten Speicher freigeben
	if (m_pucEntrance2ListMem[num]) {

		delete[] m_pucEntrance2ListMem[num];
		m_pucEntrance2ListMem[num] = NULL;
	}

	// Speicher reservieren
	m_pucEntrance2ListMem[num] = new unsigned char[size + sizeof(unsigned long)];

	if (m_pucEntrance2ListMem[num] == NULL) {

		return -IV_ERROR_LIST_PROCESSING;
	}

	// Rohe Listendaten kopieren
	for (i=0; i < size; i++) {

		m_pucEntrance2ListMem[num][sizeof(unsigned long) + i] = buf[i]; // Platz fuer Anzahl Zeilen freihalten
	}


	// Anzahl Listenzeilen eintragen
	*(unsigned long*)&m_pucEntrance2ListMem[num][0] = recordCount;
	//ChangeOrderLong(&m_pucEntranceListMem[ListNum][0]);
	
	// Listenspeichergroesse setzen
	m_ulEntrance2ListSize[num] = sizeof(unsigned long) + size;

	return IV_ERROR_SUCCESS;
}

void CAEBuffer::QuicksortEntrance2Record(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum)
{
	long lHelp1, lHelp2;

	if (lRightRecordNum > lLeftRecordNum) {

		//CString strTmp((const unsigned char*)&m_pucEntrance2ListMem[ucNum][lRightRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs]);
		lHelp1 = lLeftRecordNum - 1;
		lHelp2 = lRightRecordNum;

		for (;;) {
		
			//while (CString((const unsigned char*)&m_pucEntrance2ListMem[ucNum][++lHelp1 * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) < strTmp);
			//while (CString((const unsigned char*)&m_pucEntrance2ListMem[ucNum][--lHelp2 * m_ucEntranceRecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) > strTmp);

			while (strcmp((const char*)&m_pucEntrance2ListMem[ucNum][++lHelp1 * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				          (const char*)&m_pucEntrance2ListMem[ucNum][lRightRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) < 0);
			while (strcmp((const char*)&m_pucEntrance2ListMem[ucNum][--lHelp2 * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				          (const char*)&m_pucEntrance2ListMem[ucNum][lRightRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) > 0);

			if (lHelp1 >= lHelp2) {
			
				break;
			}

			if (strcmp((const char*)&m_pucEntrance2ListMem[ucNum][lHelp1 * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				       (const char*)&m_pucEntrance2ListMem[ucNum][lHelp2 * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) != 0) {
			
				SwapEntrance2Record(ucNum, lHelp1, lHelp2);
			}
		}

		if (strcmp((const char*)&m_pucEntrance2ListMem[ucNum][lHelp1 * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs], 
				   (const char*)&m_pucEntrance2ListMem[ucNum][lRightRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + ucOffs]) != 0) {

			SwapEntrance2Record(ucNum, lHelp1, lRightRecordNum);
		}
		QuicksortEntrance2Record(ucNum, ucOffs, lLeftRecordNum, lHelp1 - 1);
		QuicksortEntrance2Record(ucNum, ucOffs, lHelp1 + 1, lRightRecordNum);
	}
}

void CAEBuffer::SwapEntrance2Record(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum)
{
	int i;
	unsigned char ucTmp;
	for (i=0; i < m_ucEntrance2RecordSize[ucNum]; i++) {
	
		ucTmp = m_pucEntrance2ListMem[ucNum][lLeftRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + i];
		m_pucEntrance2ListMem[ucNum][lLeftRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + i] = m_pucEntrance2ListMem[ucNum][lRightRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + i];
		m_pucEntrance2ListMem[ucNum][lRightRecordNum * m_ucEntrance2RecordSize[ucNum] + sizeof(unsigned long) + i] = ucTmp;
	}
}


void CAEBuffer::CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned long Add,int Cnt)
{
	int i;

	ASSERT(Add >= FLASH_KONFIG_START);

	for (i=0; i < Cnt; i++) {
		
		Des[i] = Src[Add + i - FLASH_KONFIG_START];
	}
}

unsigned long CAEBuffer::ChangeOrderULong(unsigned long val)
{
	unsigned char buf[4];

	*(unsigned long *)buf = val;
	ChangeOrderLong(buf);
	return *(unsigned long *)buf;
} 

int CAEBuffer::GetTableCount(unsigned char ucList)
{
	if (ucList) {
		
		return MaxListB;
	} else {
		
		return MaxDatB;
	}
}

int CAEBuffer::GetTableRecord(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount, unsigned char *pucVariable, unsigned char ucList)
{
	int i;

	if (ucList) {

		// Auf Zutrtittskontrolllisten verzweigen
		if (ucNum >= 100 && ucNum < 104) {
			// Version 1
			*pucVariable = FALSE;
			return GetTableRecordEntrance(ucNum, szName, pucFieldCount);
		} else if (ucNum >= 200 && ucNum < 207) {
			// Version 2
			*pucVariable = FALSE;
			return GetTableRecordEntrance2(ucNum, szName, pucFieldCount);
		}

		if (ucNum >= MaxListB) {
			
			return FALSE;
		}
		
		for (i=0; i < (MAX_NAME - 1) && 
			ListenBeschreibung[ucNum].Name[i] != 0; i++) {

			szName[i] = ListenBeschreibung[ucNum].Name[i];
		}
		szName[i] = 0;

		*pucFieldCount = ListenBeschreibung[ucNum].Felder;
		*pucVariable = FALSE;
	} else {

		if (ucNum >= MaxDatB) {
			
			return FALSE;
		}

		for (i=0; i < (MAX_NAME - 1) && 
			DatensatzBeschreibung[ucNum].Name[i] != 0; i++) {

			szName[i] = DatensatzBeschreibung[ucNum].Name[i];
		}
		szName[i] = 0;

		*pucFieldCount = DatensatzBeschreibung[ucNum].Felder;
	}

	return 1;
}

int CAEBuffer::GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength, unsigned char ucList)
{
	int i;

	if (ucList) {

		// Auf Zutrittskontrolllisten verzweigen
		if (ucTableNum >= 100 && ucTableNum < 104) {
			// Version 1
			return GetTableFieldEntrance(ucTableNum, ucFieldNum, szName, pucType, pucLength);
		} else if (ucTableNum >= 200 && ucTableNum < 207) {
			// Version 2
			return GetTableFieldEntrance2(ucTableNum, ucFieldNum, szName, pucType, pucLength);
		}

		if (ucTableNum >= MaxListB) {
			
			return FALSE;
		}

		if (ListenBeschreibung[ucTableNum].Felder <= ucFieldNum) {
			
			return FALSE;
		}

		for (i=0; i < (MAX_NAME - 1) && 
			ListenFeld[ucTableNum][ucFieldNum].Name[i] != 0; i++) {

			szName[i] = ListenFeld[ucTableNum][ucFieldNum].Name[i];
		}
		szName[i] = 0;

		*pucType   = ListenFeld[ucTableNum][ucFieldNum].Typ;
		*pucLength = ListenFeld[ucTableNum][ucFieldNum].Cnt;
	} else {

		if (ucTableNum >= MaxDatB) {
			
			return FALSE;
		}

		if (DatensatzBeschreibung[ucTableNum].Felder <= ucFieldNum) {
			
			return FALSE;
		}

		for (i=0; i < (MAX_NAME - 1) && 
			DatensatzFeld[ucTableNum][ucFieldNum].Name[i] != 0; i++) {

			szName[i] = DatensatzFeld[ucTableNum][ucFieldNum].Name[i];
		}
		szName[i] = 0;

		*pucType   = DatensatzFeld[ucTableNum][ucFieldNum].Typ;
		*pucLength = DatensatzFeld[ucTableNum][ucFieldNum].Cnt;
	}

	return 1;
}

int CAEBuffer::GetTableRecordEntrance(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount)
{
	switch(ucNum - 100) {
	case 0:
		strcpy((char*)szName, "Zeitzonen");
		*pucFieldCount = 6;
		break;
	case 1:
		strcpy((char*)szName, "Zeitmodelle");
		*pucFieldCount = 16;
		break;
	case 2:
		strcpy((char*)szName, "Personalstamm");
		*pucFieldCount = 3;
		break;
	case 3:
		strcpy((char*)szName, "Feiertage");
		*pucFieldCount = 5;
		break;
	default:
		return FALSE;
	}

	return TRUE;
}

int CAEBuffer::GetTableFieldEntrance(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength)
{
	switch(ucTableNum - 100) {
	case 0: // Zeitzonen
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "WTAG");
			*pucType   = 3;
			*pucLength = 8;
			break;
		case 1:
			strcpy((char*)szName, "Start (h)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 2:
			strcpy((char*)szName, "Start (min)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 3:
			strcpy((char*)szName, "Ende (h)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 4:
			strcpy((char*)szName, "Ende (min)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 5:
			strcpy((char*)szName, "Aktiv");
			*pucType   = 3;
			*pucLength = 2;
			break;
		}
		break;
	case 1: // Zeitmodelle
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "BGR");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 1:	case 2:	case 3:	case 4:	case 5:	case 6:	case 7:	case 8:	
		case 9:	case 10: case 11: case 12: case 13: case 14: case 15: case 16:
			sprintf((char*)szName, "ZZ%i", ucFieldNum);
			*pucType   = 3;
			*pucLength = 3;
			break;
		default:
			return FALSE;
		}
		break;
	case 2: // Personalstamm
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "Ausweis_Nr");
			*pucType   = 4;
			*pucLength = 14;
			break;
		case 1:
			strcpy((char*)szName, "BGR");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 2:
			strcpy((char*)szName, "Prio");
			*pucType   = 3;
			*pucLength = 2;
			break;
		default:
			return FALSE;
		}
		break;
	case 3: // Feiertage
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "BGR");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 1:
			strcpy((char*)szName, "Tag");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 2:
			strcpy((char*)szName, "Monat");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 3:
			strcpy((char*)szName, "Jahr");
			*pucType   = 3;
			*pucLength = 5;
			break;
		case 4:
			strcpy((char*)szName, "Prio");
			*pucType   = 3;
			*pucLength = 2;
			break;
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}

	return TRUE;
}

int CAEBuffer::GetTableRecordEntrance2(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount)
{
	// Gueltigkeitsbereich pruefen
	if (!(ucNum >= 200 && ucNum < 207)) {

		return FALSE;
	}
	// Erkennungsoffset abziehen
	ucNum -= 200;

	// Werte uebernehmen
	strcpy((char*)szName, m_entrance2Table[ucNum].name);
	*pucFieldCount = m_entrance2Table[ucNum].count;

	return TRUE;
}

int CAEBuffer::GetTableFieldEntrance2(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength)
{
	// Gueltigkeitsbereich pruefen
	if (!(ucTableNum >= 200 && ucTableNum < 207)) {

		return FALSE;
	}
	// Erkennungsoffset abziehen
	ucTableNum -= 200;
/*
	// Gueltigkeitsbereich pruefen
	if (!((ucFieldNum >= 0) && (ucFieldNum < m_entrance2Table[ucTableNum].count))) {

		return FALSE;
	}
*/	
	// Werte uebernehmen
	strcpy((char*)szName, m_entrance2TableField[ucTableNum][ucFieldNum].name);
	*pucType = m_entrance2TableField[ucTableNum][ucFieldNum].type;
	*pucLength = m_entrance2TableField[ucTableNum][ucFieldNum].size;

	return TRUE;
}

void CAEBuffer::DefaultIOModule(TIOModule *pIOModule)
{
	int i;
	pIOModule->ucActiveU = 0x00;
	pIOModule->ucOnScreenU = 0x00;
	pIOModule->ucActiveI = 0x00;
	pIOModule->ucOnScreenI = 0x00;

	// Setzen der default Werte.
	for (i=0; i < 8; i++) {
		// Spannungskanaele
		sprintf((char*)pIOModule->szNameU[i], "U%i", i + 1);
		sprintf((char*)pIOModule->szUnitU[i], "V");
		pIOModule->fMinU[i] = 0;
		pIOModule->fMaxU[i] = 10;

		// Stomkanaele
		sprintf((char*)pIOModule->szNameI[i], "I%i", i + 1);
		sprintf((char*)pIOModule->szUnitI[i], "mA");
		pIOModule->fMinI[i] = 0;
		pIOModule->fMaxI[i] = 20;
	}
}

// Groesse der Setupdaten im Flash ermitteln
unsigned long CAEBuffer::GetFlashSetupSize()
{
	// ##############################################################
	// Pointer sind direkte Adressen die im AEIII+:
	//   1. Im Motorolla - Format vorliegen.
	//   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

	// Pointer auf ListenBeschreibungen ermitteln
	unsigned long pLB = *(unsigned long*)(&m_pucEepMem[P_LIST_STRUCT]);

	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pLB);
	//}
	pLB -= FLASH_KONFIG_START;

	// Pointerarray der Listenbeschreibungen durchschreiten
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
	
		// Nullpointer suchen fuer Listenanzahl
		if (*(unsigned long*)&m_pucFlashMem[pLB + i * sizeof(unsigned long)] == 0ul) {
		
			break;
		}
	}

	// Liste gefunden?
	if (i > 0) {
	
		// Pointer auf letzte definierte Liste
		pLB = *(unsigned long*)(&m_pucFlashMem[pLB + (i - 1) * sizeof(unsigned long)]);

		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pLB);
		//}
		pLB -= FLASH_KONFIG_START;

		TDatensatzBeschreibung *pLBStruct = (TDatensatzBeschreibung*)(&m_pucFlashMem[pLB]);

		// Pointer auf Feldarray der Liste
		unsigned long pFeldArray = pLBStruct->P;
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeldArray);
		//}
		pFeldArray -= FLASH_KONFIG_START;

		// Felder vorhanden?
		if (pLBStruct->Felder) {
		
			// Pointer auf letztes Feld der Listenbeschreibung ermitteln.
			unsigned long pFeld = *(unsigned long*)&m_pucFlashMem[pFeldArray + (pLBStruct->Felder - 1) * sizeof(unsigned long)];
			//if (App->ByteOrder == BO_MOT) {
				
				ChangeOrderLong((unsigned char*)&pFeld);
			//}
			pFeld -= FLASH_KONFIG_START;
			
			// Feldadresse + eigene Groesse bildet FlashSetupSize
			return pFeld + sizeof(TDatensatzFeld);
		}


		// Keine Felder definiert!
		return pFeldArray;
	}
	
	// Keine Listenbeschreibung definiert!
	// Die Anzahl fuer Listenbeschreibungspointer muss mitberechnet werden,
	// um bei einer Uebertragung diese zu loeschen.
	return pLB + MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned long);
}

unsigned long CAEBuffer::GetMDEEKFlashSize_0400()
{
	// Erweiterten Struktur lesen
	TExtendedFlash extBegin = *(TExtendedFlash*)m_pucExtendedMem;
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer MDE lesen.
	if (extBegin.pulEKArray == 0) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 1Byte Anzahl Eingabeketten fuer MDE
		return 1;
	
	}

	int Unterpunkte;
	unsigned long pEK;

	Unterpunkte = m_pucExtendedMem[extBegin.pulEKArray];
	if (Unterpunkte == 0) {
	
		// 1Byte Anzahl Eingabeketten fuer MDE
		return 1;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArray + sizeof(unsigned char) + (Unterpunkte - 1) * sizeof(unsigned long);	
	pEK = *(unsigned long*)(&m_pucExtendedMem[pEK]);
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// 16kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
	pEK -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pEK];

	// Pointer auf Feldarray der Eingabekette
	unsigned long pFeldArray = pDE->P;
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pFeldArray);
	//}
	pFeldArray -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		unsigned long pFeld = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned long)];
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeld);
		//}
		pFeld -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

unsigned long CAEBuffer::GetMDEEKFlashSize()
{
	// Erweiterten Struktur lesen
	TExtendedFlash extBegin = *(TExtendedFlash*)m_pucExtendedMem;
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer MDE lesen.
	if (extBegin.pulEKArray == 0) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 4Byte Anzahl Eingabeketten fuer MDE
		return 4;
	
	}

	int Unterpunkte;
	unsigned long pEK;

	Unterpunkte = m_pucExtendedMem[extBegin.pulEKArray];
	if (Unterpunkte == 0) {
	
		// 4Byte Anzahl Eingabeketten fuer MDE
		return 4;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArray + sizeof(unsigned long) + (Unterpunkte - 1) * sizeof(unsigned long);	
	pEK = *(unsigned long*)(&m_pucExtendedMem[pEK]);
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// 16kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
	pEK -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pEK];

	// Pointer auf Feldarray der Eingabekette
	unsigned long pFeldArray = pDE->P;
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pFeldArray);
	//}
	pFeldArray -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		unsigned long pFeld = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned long)];
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeld);
		//}
		pFeld -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

unsigned long CAEBuffer::GetEntranceEKFlashSize()
{
	// Erweiterten Struktur lesen
	TExtendedFlash extBegin = *(TExtendedFlash*)m_pucExtendedMem;
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer Zutritt lesen.
	if (extBegin.pulEKArrayEntrance == 0) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return 4;
	
	}

	int Unterpunkte;
	unsigned long pEK;

	Unterpunkte = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance];
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&Unterpunkte);
	//}
	if (Unterpunkte == 0) {
	
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return 4;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArrayEntrance + sizeof(unsigned long) + (Unterpunkte - 1) * sizeof(unsigned long);	
	pEK = *(unsigned long*)(&m_pucExtendedMem[pEK]);
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// 32kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
	pEK -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance + pEK];

	// Pointer auf Feldarray der Eingabekette
	unsigned long pFeldArray = pDE->P;
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pFeldArray);
	//}
	pFeldArray -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		unsigned long pFeld = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned long)];
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeld);
		//}
		pFeld -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

unsigned long CAEBuffer::GetEntrance2EKFlashSize()
{
	// Erweiterten Struktur lesen
	TExtendedFlash extBegin = *(TExtendedFlash*)m_pucExtendedMem;
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer Zutritt lesen.
	if (extBegin.pulEKArrayEntrance2 == 0) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return 4;
	
	}

	int Unterpunkte;
	unsigned long pEK;

	Unterpunkte = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2];
	//if (theApp.ByteOrder == BO_MOT) {
		
//		ChangeOrderLong((unsigned char*)&Unterpunkte);
	//}
	if (Unterpunkte == 0) {
	
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return 4;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArrayEntrance2 + sizeof(unsigned long) + (Unterpunkte - 1) * sizeof(unsigned long);	
	pEK = *(unsigned long*)(&m_pucExtendedMem[pEK]);
	//if (theApp.ByteOrder == BO_MOT) {
		
//		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// Basisadresse fuer Konfigdaten subtrahieren
	pEK -= FLASH_KONFIG_START;

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + pEK];

	// Pointer auf Feldarray der Eingabekette
	unsigned long pFeldArray = pDE->P;
	//if (theApp.ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pFeldArray);
	//}
	pFeldArray -= FLASH_KONFIG_START;

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		unsigned long pFeld = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned long)];
		//if (theApp.ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeld);
		//}
		pFeld -= FLASH_KONFIG_START;
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&ExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

VOID CAEBuffer::PrepareEntrance2EK(unsigned long offs, unsigned char *buf)
{
	// Erweiterten Struktur lesen
	int num, numFeld, posPointer = 0, posData = 0, posFeldArray;
	TExtendedFlash extBegin = *(TExtendedFlash*)m_pucExtendedMem;
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return;
	}

	// Sind Keine Angaben zur ZK vorhanden?
	if (extBegin.pulEKArrayEntrance2 == 0) {

		// Keine Unterpunkte
		*(unsigned long*)buf = 0;
		return;
	}

	int Unterpunkte;
	unsigned long pEK;

	Unterpunkte = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2];
	//if (theApp.ByteOrder == BO_MOT) {
		
//		ChangeOrderLong((unsigned char*)&pEK);
	//}
	if (Unterpunkte == 0) {
	
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return;
	}

	// Anzahl Eingabeketten eintragen
	*(unsigned long*)buf = Unterpunkte;
	posPointer = sizeof(unsigned long);
	posData    = sizeof(unsigned long) + Unterpunkte * sizeof(unsigned long);;
	
	// Alle Eingabeketten durchlaufen
	for (num=0; num < Unterpunkte; num++) {

		// Pointer der Eingabekette ermitteln
		pEK = extBegin.pulEKArrayEntrance2 + sizeof(unsigned long) + num * sizeof(unsigned long);	
		pEK = *(unsigned long*)&m_pucExtendedMem[pEK];
		//if (theApp.ByteOrder == BO_MOT) {
			
//			ChangeOrderLong((unsigned char*)&pEK);
		//}
		// Basisadresse fuer Konfigdaten subtrahieren
		pEK -= FLASH_KONFIG_START;

		// Zeiger auf Eingabekette uebernehmen (ohne Basisoffset: FLASH_KONFIG_START)
		*(unsigned long*)&buf[posPointer] = pEK;
		posPointer += sizeof(unsigned long);


		// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
		// diese uebernehmen
		memcpy(&buf[posData], &m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + pEK], sizeof(TBedien));
		posData += sizeof(TBedien);
		// dann uebergehen
		pEK += sizeof(TBedien);

		// Zeiger auf Struktur 
		TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + pEK];

		// Pointer auf Feldarray der Eingabekette
		unsigned long pFeldArray = pDE->P;
		//if (theApp.ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeldArray);
		//}
		pFeldArray -= FLASH_KONFIG_START;

		// Zeiger auf Feldarray anpassen und Struktur uebernehmen
		ChangeOrderLong((unsigned char*)&pDE->P);
		pDE->P += offs;
		ChangeOrderLong((unsigned char*)&pDE->P);
		memcpy(&buf[posData], pDE, sizeof(TBDatensatzEinlesen));
		ChangeOrderLong((unsigned char*)&pDE->P);
		pDE->P -= offs;
		ChangeOrderLong((unsigned char*)&pDE->P);
		posData += sizeof(TBDatensatzEinlesen);

		posFeldArray = posData;
		posData += pDE->Felder * sizeof(unsigned long);
		// Felder abarbeiten
		for (numFeld=0; numFeld < pDE->Felder; numFeld++) {
		
			// Pointer auf Feld der Eingabekette ermitteln.
			unsigned long pFeld = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + pFeldArray + numFeld * sizeof(unsigned long)];
			//if (theApp.ByteOrder == BO_MOT) {
				
				ChangeOrderLong((unsigned char*)&pFeld);
			//}

			// Zeiger auf Feldarray anpassen und Struktur uebernehmen
			*(unsigned long*)&buf[posFeldArray] = pFeld + offs;
			ChangeOrderLong(&buf[posFeldArray]);
			posFeldArray += sizeof(unsigned long);

			pFeld -= FLASH_KONFIG_START;

			TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + pFeld];
			
			// Feldstruktur uebernehmen
			memcpy(&buf[posData], pDF, sizeof(TBFeldEinlesen));
			// Da die Strukturen der Felder hintereinander liegen ist zwischen den
			// Zeigern immer ein Unterschied von sizeof(TBFeldEinlesen)
			posData += sizeof(TBFeldEinlesen);
		}
	}
}

void CAEBuffer::AdjustExtendedMemPointer()
{
	// Erweiterten Struktur lesen
	unsigned long num;
	int numFeld;
	TExtendedFlash extBegin = *(TExtendedFlash*)m_pucExtendedMem;
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return;
	}
	
	unsigned long count;
	unsigned long rootOffset, childOffset, tmp, tmp2;
	///////////////////////////////////////////////////////
	// MDE
	if (extBegin.pulEKArray != 0) {

		// Zeiger der Eingabeketten fuer MDE anpassen
		count = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray];
		ChangeOrderLong((unsigned char*)&count);
		
		// Jede Eingabekette anspringen
		for (num=0; num < count; num++) {
			
			rootOffset = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray + (num + 1) * sizeof(unsigned long)];
			ChangeOrderLong((unsigned char*)&rootOffset);
			
			tmp = rootOffset;
			
			if (rootOffset > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
				
				rootOffset -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
				tmp -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
			} else {
				
				rootOffset += (FLASH_SIZE_SU + FLASH_KONFIG_START);
			}
			
			ChangeOrderLong((unsigned char*)&rootOffset);
			*(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray + (num + 1) * sizeof(unsigned long)] = rootOffset;
			
			TBedien bedien = *(TBedien*)&m_pucExtendedMem[extBegin.pulEKArray + tmp];
			ChangeOrderLong((unsigned char*)&bedien.P);
			
			tmp2 = bedien.P;
			
			if (bedien.P > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
				
				bedien.P -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
				tmp2 -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
			} else {
				
				bedien.P += (FLASH_SIZE_SU + FLASH_KONFIG_START);
			}
			
			ChangeOrderLong((unsigned char*)&bedien.P);
			*(TBedien*)&m_pucExtendedMem[extBegin.pulEKArray + tmp] = bedien;
			
			TBDatensatzEinlesen datensatzEinlesen = *(TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + tmp2];
			ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
			
			tmp = datensatzEinlesen.P;
			
			if (datensatzEinlesen.P > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
				
				datensatzEinlesen.P -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
				tmp -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
			} else {
				
				datensatzEinlesen.P += (FLASH_SIZE_SU + FLASH_KONFIG_START);
			}
			
			ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
			*(TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + tmp2] = datensatzEinlesen;
			
			for (numFeld=0; numFeld < datensatzEinlesen.Felder; numFeld++) {
				
				childOffset = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray + tmp + numFeld * sizeof(unsigned long)];
				ChangeOrderLong((unsigned char*)&childOffset);
				
				if (childOffset > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
					
					childOffset -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
				} else {
					
					childOffset += (FLASH_SIZE_SU + FLASH_KONFIG_START);
				}
				
				ChangeOrderLong((unsigned char*)&childOffset);
				*(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArray + tmp + numFeld * sizeof(unsigned long)] = childOffset;
			}
		}
	}

	///////////////////////////////////////////////////////
	// Zutrittskontrolle Version 2

	/////////////////////////
	// Tabellen
	if (extBegin.pulEntrance2Tables != 0) {

		// Zeiger der Tabellen fuer ZK anpassen
		count = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEntrance2Tables];
		
		// Jede Tabelle anspringen
		for (num=0; num < count; num++) {
			
			rootOffset = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEntrance2Tables + (num + 1) * sizeof(unsigned long)];
			// Die Zeiger auf die einzelnen Tabellen sind 
			tmp = rootOffset;
			
			if (rootOffset > FLASH_KONFIG_START) {
				
				rootOffset -= FLASH_KONFIG_START;
				tmp -= FLASH_KONFIG_START;
			} else {
				
				rootOffset += FLASH_KONFIG_START;
			}
			
			*(unsigned long*)&m_pucExtendedMem[extBegin.pulEntrance2Tables + (num + 1) * sizeof(unsigned long)] = rootOffset;
			
			TDatensatzBeschreibung datensatzBeschreibung = *(TDatensatzBeschreibung*)&m_pucExtendedMem[extBegin.pulEntrance2Tables + tmp];
			ChangeOrderLong((unsigned char*)&datensatzBeschreibung.P);
			
			tmp2 = datensatzBeschreibung.P;
			
			if (datensatzBeschreibung.P > FLASH_KONFIG_START) {
				
				datensatzBeschreibung.P -= FLASH_KONFIG_START;
				tmp2 -= FLASH_KONFIG_START;
			} else {
				
				datensatzBeschreibung.P += FLASH_KONFIG_START;
			}
			
			ChangeOrderLong((unsigned char*)&datensatzBeschreibung.P);
			*(TDatensatzBeschreibung*)&m_pucExtendedMem[extBegin.pulEntrance2Tables + tmp] = datensatzBeschreibung;
			
			// Zeiger auf die Felder abarbeiten
			for (numFeld=0; numFeld < datensatzBeschreibung.Felder; numFeld++) {
				
				childOffset = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEntrance2Tables + tmp2 + numFeld * sizeof(unsigned long)];
				ChangeOrderLong((unsigned char*)&childOffset);
				
				if (childOffset > FLASH_KONFIG_START) {
					
					childOffset -= FLASH_KONFIG_START;
				} else {
					
					childOffset += FLASH_KONFIG_START;
				}
				
				ChangeOrderLong((unsigned char*)&childOffset);
				*(unsigned long*)&m_pucExtendedMem[extBegin.pulEntrance2Tables + tmp2 + numFeld * sizeof(unsigned long)] = childOffset;
			}
		}
	}

	/////////////////////////
	// Eingabeketten
	if (extBegin.pulEKArrayEntrance2 != 0) {

		// Zeiger der Eingabeketten fuer MDE anpassen
		count = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2];
		
		// Jede Eingabekette anspringen
		for (num=0; num < count; num++) {
			
			rootOffset = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + (num + 1) * sizeof(unsigned long)];
			
			tmp = rootOffset;
			
			if (rootOffset > FLASH_KONFIG_START) {
				
				rootOffset -= FLASH_KONFIG_START;
				tmp -= FLASH_KONFIG_START;
			} else {
				
				rootOffset += FLASH_KONFIG_START;
			}
			
			*(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + (num + 1) * sizeof(unsigned long)] = rootOffset;
			
			TBedien bedien = *(TBedien*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + tmp];
			ChangeOrderLong((unsigned char*)&bedien.P);
			
			tmp2 = bedien.P;
			
			if (bedien.P > FLASH_KONFIG_START) {
				
				bedien.P -= FLASH_KONFIG_START;
				tmp2 -= FLASH_KONFIG_START;
			} else {
				
				bedien.P += FLASH_KONFIG_START;
			}
			
			ChangeOrderLong((unsigned char*)&bedien.P);
			*(TBedien*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + tmp] = bedien;
			
			TBDatensatzEinlesen datensatzEinlesen = *(TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + tmp2];
			ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
			
			tmp = datensatzEinlesen.P;
			
			if (datensatzEinlesen.P > FLASH_KONFIG_START) {
				
				datensatzEinlesen.P -= FLASH_KONFIG_START;
				tmp -= FLASH_KONFIG_START;
			} else {
				
				datensatzEinlesen.P += FLASH_KONFIG_START;
			}
			
			ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
			*(TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + tmp2] = datensatzEinlesen;
			
			// Zeiger auf die Felder abarbeiten
			for (numFeld=0; numFeld < datensatzEinlesen.Felder; numFeld++) {
				
				childOffset = *(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + tmp + numFeld * sizeof(unsigned long)];
				ChangeOrderLong((unsigned char*)&childOffset);
				
				if (childOffset > FLASH_KONFIG_START) {
					
					childOffset -= FLASH_KONFIG_START;
				} else {
					
					childOffset += FLASH_KONFIG_START;
				}
				
				ChangeOrderLong((unsigned char*)&childOffset);
				*(unsigned long*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance2 + tmp + numFeld * sizeof(unsigned long)] = childOffset;
			}
		}
	}
}

void CAEBuffer::Entrance2Default()
{
	int table, field;

	// Pointerarray auf Strukturen initialisieren
	memset(m_entrance2Pointer, 0, sizeof(m_entrance2Pointer));

	// List: Reader
	table = 0;
	m_entrance2Pointer[table] = MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned long);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Reader");
	m_entrance2Table[table].count = 6;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = TF_SORT;

// Id	Nummer	4	Int	Der primre Schlssel der Reader - Tabelle. Er muss eindeutig sein.
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Id");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// ZM	Nummer	4	Int	Gibt die zugehrige Master ID an.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "ZM");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// TM	Nummer	3	Int	Gibt die Busnummer des Leser an.(intern 010..160, extern 011..161)
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "TM");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 4;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefLocation	Nummer	4	Int	Definiert die Raum ID, zu dem der ZL Zutritt verschafft.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefLocation");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefAction	Nummer	4	Int	Aktion, die bei gltigem Zutritt ausgelst wird.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefAction");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));
	
// PinGeneral	Nummer	8	Int	Mit der hier hinterlegten PIN erhlt man ohne AW Zutritt.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "PinGeneral");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 9;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));
	

	// List: Identification
	table++;
	m_entrance2Pointer[table] = m_entrance2Pointer[table - 1] + sizeof(STable2) + ++field * sizeof(STable2Field);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Identification");
	m_entrance2Table[table].count = 7;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = TF_SORT;
	
// Id	Text	20	ASCII	Gibt die zu prfende Ausweisnummer an.
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Id");
	m_entrance2TableField[table][field].type = TF_A_NUM;
	m_entrance2TableField[table][field].size = 21;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// Group	Nummer	4	Int	Gibt die BG an, in welcher der Ausweis mit anderen enthalten ist.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Group");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// Pin	Nummer	8	Int	Definiert die PIN. Falls 0 dann keine Abfrage
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Pin");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 9;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// Menace	Nummer	4	Int	Definiert die Bedrohungsziffern (Erweiterungsziffern zum PIN). Falls 0 dann keine Abfrage
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Menace");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// ActiveStart	Text	10	Date	Sperrbereich des AW (Ab wann ist der AW gltig).
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "ActiveStart");
	m_entrance2TableField[table][field].type = TF_A_NUM;
	m_entrance2TableField[table][field].size = 11;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// ActiveEnd	Text	10	Date	Sperrbereich des AW (Bis wann ist der AW gltig).
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "ActiveEnd");
	m_entrance2TableField[table][field].type = TF_A_NUM;
	m_entrance2TableField[table][field].size = 11;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// ActiveGeneral	Nummer	1	Int	Globale Sperrung des AW.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "ActiveGeneral");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 2;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));



	// List: Location
	table++;
	m_entrance2Pointer[table] = m_entrance2Pointer[table - 1] + sizeof(STable2) + ++field * sizeof(STable2Field);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Location");
	m_entrance2Table[table].count = 4;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = TF_SORT;
	
// Id	Nummer	4	Int	Gibt den Raum an.
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Id");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefGroup	Nummer	4	Int	Die BG fr den Raum.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefGroup");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefTime	Nummer	4	Int	Das entsprechende Zeitmodell.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefTime");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefTimePin		Nummer	4	Int	Zeitmodell, wann eine PIN eingegeben werden mu.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefTimeNoPin");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));



	// List: Time
	table++;
	m_entrance2Pointer[table] = m_entrance2Pointer[table - 1] + sizeof(STable2) + ++field * sizeof(STable2Field);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Time");
	m_entrance2Table[table].count = 4;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = TF_SORT;

// Id	Nummer	4	Int	ID des Zeitmodell.
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Id");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// Weekdays	Nummer	7	Weekday	Definiert die Wochentage, an den das Zeitmodell gilt. (Format: Max. 7 Stellen 1 - 7 z.B. 134567= Montag, Mittwoch bis Sonntag)
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Weekdays");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 8;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// TimeStart	Text	5	Time	Von wann gltig. (Format 24h: HH:MM)
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "TimeStart");
	m_entrance2TableField[table][field].type = TF_A_NUM;
	m_entrance2TableField[table][field].size = 6;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// TimeEnd	Text	5	Time	Bis wann gltig.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "TimeEnd");
	m_entrance2TableField[table][field].type = TF_A_NUM;
	m_entrance2TableField[table][field].size = 6;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));



	// List: Holiday
	table++;
	m_entrance2Pointer[table] = m_entrance2Pointer[table - 1] + sizeof(STable2) + ++field * sizeof(STable2Field);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Holiday");
	m_entrance2Table[table].count = 3;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = 0;

// Day	Text	10	Date	Datum des Sperrtages. (Format: YYYY-MM-DD)
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Day");
	m_entrance2TableField[table][field].type = TF_A_NUM;
	m_entrance2TableField[table][field].size = 11;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefGroup	Nummer	4	Int	Gibt die BG fr den Sperrtag an.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefGroup");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefTime	Nummer	4	Int	Gibt das abweichende Zeitmodell an.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefTime");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));



	// List: Event
	table++;
	m_entrance2Pointer[table] = m_entrance2Pointer[table - 1] + sizeof(STable2) + ++field * sizeof(STable2Field);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Event");
	m_entrance2Table[table].count = 4;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = 0;

// RefReader	Nummer	4	Int	Master, an dem das TM ist.
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefReader");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// PortIn	Nummer	1	Int	Gibt den DI am TM an.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "PortIn");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 2;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefAction	Nummer	4	Int	ID der Aktion, die ausgefhrt werden soll.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefAction");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefTime	Nummer	4	Int	Zeitmodell, wann der DI gltig ist.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefTime");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));



	// List: Action
	table++;
	m_entrance2Pointer[table] = m_entrance2Pointer[table - 1] + sizeof(STable2) + ++field * sizeof(STable2Field);

	m_entrance2Table[table].num = table;
	strcpy(m_entrance2Table[table].name, "Action");
	m_entrance2Table[table].count = 5;
	m_entrance2Table[table].key = 0;
	m_entrance2Table[table].sort = 0;
	m_entrance2Table[table].flags = TF_SORT;

// Id	Nummer	4	Int	ID der Aktion.
	field = 0;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Id");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefReader	Nummer	4	Int	Master, an dem das Ereignis ausgelst werden soll.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefReader");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// PortOut	Nummer	1	Int	Welcher Ausgang wird geschaltet.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "PortOut");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 2;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// Elapse	Nummer	3	Int	Dauer des Schaltens in Sekunden, 0 -> dauernd
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "Elapse");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 4;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

// RefTime	Nummer	4	Int	Zeitmodell, wann das Relais auslsen soll.
	field++;
	m_entrance2TableField[table][field].num = field;
	strcpy(m_entrance2TableField[table][field].name, "RefTime");
	m_entrance2TableField[table][field].type = TF_NUM;
	m_entrance2TableField[table][field].size = 5;
	m_entrance2TableField[table][field].flags = 0;
	memset(m_entrance2TableField[table][field].format, 0, sizeof(m_entrance2TableField[table][field].format));

}

void CAEBuffer::ClearTimeboyListBuffer(unsigned char id, unsigned char num)
{
	unsigned char groupID, listNum;

	if (id == 0xFF) {

		for (groupID=0; groupID < MAX_DATENSATZ_BESCHREIBUNG; groupID++) {
			if (num == 0xFF) { // Alle loeschen

				for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++) {

					m_ulTimeboyListSize[groupID][listNum] = 0;
					m_timeboyFlags[groupID][listNum] = 0;

					// Reservierten Speicher freigeben
					if (m_pucTimeboyListMem[groupID][listNum]) {

						delete[] m_pucTimeboyListMem[groupID][listNum];
						m_pucTimeboyListMem[groupID][listNum] = NULL;
					}
				}

			} else {

				if (num >= MAX_DATENSATZ_BESCHREIBUNG) return;

				m_ulTimeboyListSize[groupID][num] = 0;
				m_timeboyFlags[groupID][num] = 0;

				// Reservierten Speicher freigeben
				if (m_pucTimeboyListMem[groupID][num]) {

					delete[] m_pucTimeboyListMem[groupID][num];
					m_pucTimeboyListMem[groupID][num] = NULL;
				}
			}
		}
		
	} else {
		if (id >= MAX_DATENSATZ_BESCHREIBUNG) return;
			
		if (num == 0xFF) { // Alle loeschen

			for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++) {

				m_ulTimeboyListSize[id][listNum] = 0;
				m_timeboyFlags[id][listNum] = 0;

				// Reservierten Speicher freigeben
				if (m_pucTimeboyListMem[id][listNum]) {

					delete[] m_pucTimeboyListMem[id][listNum];
					m_pucTimeboyListMem[id][listNum] = NULL;
				}
			}

		} else {

			if (num >= MAX_DATENSATZ_BESCHREIBUNG) return;

			m_ulTimeboyListSize[id][num] = 0;
			m_timeboyFlags[id][num] = 0;

			// Reservierten Speicher freigeben
			if (m_pucTimeboyListMem[id][num]) {

				delete[] m_pucTimeboyListMem[id][num];
				m_pucTimeboyListMem[id][num] = NULL;
			}
		}
	}
}

int CAEBuffer::MakeTimeboyList(unsigned char id, unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned long size, int flags, int *error)
{
	unsigned long i;

	// Zuvor reservierten Speicher freigeben
	if (m_pucTimeboyListMem[id][num]) {

		delete[] m_pucTimeboyListMem[id][num];
		m_pucTimeboyListMem[id][num] = NULL;
	}

	// Soll die Liste geloescht werden?
	if (flags & 0x01) {

		// Sicherstellen, dass keine unnoetigen Daten versendet werden.
		size = 0;
		recordCount = 0;
	}
	// Flags uebernehmen
	m_timeboyFlags[id][num] = flags;

	// Speicher reservieren
	m_pucTimeboyListMem[id][num] = new unsigned char[size + sizeof(unsigned long)];

	if (m_pucTimeboyListMem[id][num] == NULL) {

		*error = IV_ERROR_LIST_PROCESSING;
		return FALSE;
	}

	// Rohe Listendaten kopieren
	for (i=0; i < size; i++) {

		m_pucTimeboyListMem[id][num][sizeof(unsigned long) + i] = buf[i]; // Platz fuer Anzahl Zeilen freihalten
	}

	// Anzahl Listenzeilen eintragen
	*(unsigned long*)&m_pucTimeboyListMem[id][num][0] = recordCount;
	
	// Listenspeichergroesse setzen
	m_ulTimeboyListSize[id][num] = sizeof(unsigned long) + size;

	return TRUE;
}
