/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef CHANNEL4_MODULE_SYSTEM_H
#define CHANNEL4_MODULE_SYSTEM_H

// Basisklasse einbinden
#include "Channel4ModuleHardware.h"
#include <sstream>
#include <string>
using std::string;

#include <boost/format.hpp>
using boost::str;
using boost::format;

/** In diesem Module sind alle Systemspezifischen Funktionen enthalten.
 *
 */
#define IV_MODULE_SYSTEM                         0x0001


// Ermitteln der Firmwareversion
#define IV_FUNCTION_GET_VERSION                  0x0000

// Setzen/Ermitteln der Seriennummer
#define IV_FUNCTION_SET_SERIAL                   0x0001
#define IV_FUNCTION_GET_SERIAL                   0x0002
    
// Setzen/Ermitteln der Systemzeit. Format (YYYY-MM-DD HH:MM:SS\0)
#define IV_FUNCTION_SET_RTC                      0x0003
#define IV_FUNCTION_GET_RTC                      0x0004


// Setzen/Ermitteln einer Systemvariablen 
#define IV_FUNCTION_SET_SYSTEM_VARIABLE          0x0005
#define IV_FUNCTION_GET_SYSTEM_VARIABLE          0x0006

// Der Name einer Systemvariablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Für GPRS ist der Majorname MODEM_MC35i.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Länge des Variablenwertes ist zu lang.


// Setzen/Ermitteln eines Relaiszustandes
#define IV_FUNCTION_SET_RELAY                    0x0007
#define IV_FUNCTION_GET_RELAY                    0x0008

// Die Namen sind: INTERN, TMR (Türmodule), IOM (IOModule)
// Die Busnummer ist die Busnummer bei 0 beginnend. (Bei INTERN eigentlich immer 0)
// Die Relaisnummer ist die Relaisnummer bei 0 beginnend.
// Die Zeit ist eine Angabe in Sekunden (0 bedeutet eine Dauerschaltung in den gewählten Status)
// Fehlernummer:
// 1 = Name undefiniert
// 2 = Busnummer undefiniert
// 3 = Relaisnummer undefiniert
// 4 = Status undefiniert  (0, 1) (Könnte irgendwann um andere Varianten erweitert werden)

// Schreiben/Lesen von: Externen Flash
#define IV_FUNCTION_SET_FLASH                    0x0009
#define IV_FUNCTION_GET_FLASH                    0x000A

// Schreiben/Lesen von: EEPROM
#define IV_FUNCTION_SET_EEPROM                   0x000B
#define IV_FUNCTION_GET_EEPROM                   0x000C

// Schreiben/Lesen von: Program-Flash schreiben/lesen
#define IV_FUNCTION_SET_PROGRAM                  0x000D // (wird nur aufgenommen)
#define IV_FUNCTION_GET_PROGRAM                  0x000E

// Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
#define IV_FUNCTION_BEGIN_SET_SEGMENT                0x000F
#define IV_FUNCTION_BEGIN_SET_SEGMENT_OK             0x0010
#define IV_FUNCTION_END_SET_SEGMENT                  0x0011
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_SEGMENT und _END_SEGMENT
#define IV_VALUE_SEGMENT_UPDATE                            0 // Es werden Daten gesetzt die nachdem Uebernommen werden.
#define IV_VALUE_SEGMENT_UPDATE_BG                         1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_FIRMWARE                          2
#define IV_VALUE_SEGMENT_FONTS                             3
#define IV_VALUE_SEGMENT_GRAFICS                           4
#define IV_VALUE_SEGMENT_SETUP                             5
#define IV_VALUE_SEGMENT_LIST                              6
#define IV_VALUE_SEGMENT_SETUP_EXTENDED                    7
#define IV_VALUE_SEGMENT_LIST_ACCESS                       8 // Listen fuer ZK II werden uebertragen, dadurch wird die Zutrittskontrolle blockiert.
#define IV_VALUE_SEGMENT_LIST_TIMEBOY                      9 // Liste fuer Timeboy uebertragen
#define IV_VALUE_SEGMENT_FINGERPRINT                       10
#define IV_VALUE_SEGMENT_TEXT                              11
#define IV_VALUE_SEGMENT_TOUCHKEYS                         12
#define IV_VALUE_SEGMENT_DISPLAYCOLORS                     13
#define IV_VALUE_SEGMENT_WLANCONFIGS                       14
#define IV_VALUE_SEGMENT_HARDWARECONFIGS                   15
#define IV_VALUE_SEGMENT_FILETRANSFER                      16

// 0  -> Global fuer Uebernahmerahmen von Datenbundel
// 1  -> Firmware
// 2  -> Fonts
// 3  -> Grafik
// 4  -> Setup
// 9  -> Timeboylisten uebertragen
// 10 -> Firmware des Fingerprintmoduls uebertragen
// 11 -> Segment fuer Textdaten der Firmware uebertragen
// 12 -> Segment fuer Daten der Tasten auf Touch übertragen
// 13 -> Segment fuer die Farbdaten des Displays
// 14 -> Segment fuer die WLAN Konfigurationen des Redpine Moduls
// 15 -> Segment für die Hardwarekonfiguration
// 16 -> 
// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfügung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_SET_SEGMENT_OK               0x0012
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

// Ermitteln von: Bootloaderversion
#define IV_FUNCTION_GET_VERSION_BOOTLOADER       0x0013
// Format: XX.XX.XX??????????????

// Durchfuehren eines Resets der Speicherbereiche
#define IV_FUNCTION_SYSTEM_RESET                 0x0014

// Resetmodus:
#define IV_VALUE_RESET_ALL             0 // Geraet in einen Zustand versetzen, wie nach dem Bootloader einspielen.
#define IV_VALUE_RESET_RESTART         1 // Warmstart (Watchdog ausführen)
#define IV_VALUE_RESET_FORMAT          2 // Flash formatieren.
#define IV_VALUE_RESET_SETUP           3 // Setup, Listen und Datensatzzeiger Löschen/Zurücksetzen

// Fehler:
// 1 -> Resetmodus unbekannt.


// Ins Bios wechseln
#define IV_FUNCTION_ENTER_BIOS			         0x0015
// Bioswechsel erfolgt
#define IV_FUNCTION_ENTERED_BIOS		         0x0016

// Bios verlassen (aenderungen uebernehmen
#define IV_FUNCTION_EXIT_BIOS			         0x0017
// Biosverlassen erfolgt
#define IV_FUNCTION_EXITED_BIOS			         0x0018

// Setzt/liest das zu verwendende Interface
#define IV_FUNCTION_SET_INTERFACE                0x0019
// 0 -> RS232
// 1 -> TCP/IP
// 2 -> GSM
// 3 -> GPRS
// 4 -> RS485

// 9 -> USB-Host
#define IV_FUNCTION_GET_INTERFACE                0x001A

#define IV_VALUE_INTERFACE_RS232       0
#define IV_VALUE_INTERFACE_TCPIP       1
#define IV_VALUE_INTERFACE_GSM         2
#define IV_VALUE_INTERFACE_GPRS        3
#define IV_VALUE_INTERFACE_RS485       4
#define IV_VALUE_INTERFACE_USB_HOST    9


// Setzen/Ermitteln der zu verwendende BaudRate
#define IV_FUNCTION_SET_BAUD                     0x001B
#define IV_FUNCTION_GET_BAUD                     0x001C
// Setzen/Ermitteln der zu verwendenden DeviceID
#define IV_FUNCTION_SET_DEVICE_ID                0x001D
#define IV_FUNCTION_GET_DEVICE_ID                0x001E

// Befehl der beim Geraetestart den Bootloader des MasterIV wiederherstellt.
#define IV_FUNCTION_ENTER_BOOTLOADER             0x001F

// In Monitor Mode von XPort wechselt -> siehe auch #define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT		 0x0028 
#define IV_FUNCTION_ENTER_XPORT_MONITOR          0x0020
// Antwort in Monitor Mode gewechselt -> siehe auch #define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK   0x0029
#define IV_FUNCTION_ENTER_XPORT_MONITOR_OK       0x0021

// Monitorbefehl an XPort und Antwort erhalten.
#define IV_FUNCTION_CMD_XPORT                    0x0022

// In Monitor Mode von XPort wechselt -> siehe auch #define 
#define IV_FUNCTION_EXIT_XPORT_MONITOR           0x0023
// Antwort in Monitor Mode gewechselt -> siehe auch #define 
#define IV_FUNCTION_EXIT_XPORT_MONITOR_OK        0x0024


#define IV_FUNCTION_BEGIN_GET_SEGMENT            0x0025
// [in]-> [UINT16(SS_...)]
// <- 16bit 
#define IV_FUNCTION_BEGIN_GET_SEGMENT_OK         0x0026
// [out]-> 16bit Typ des Speichers
// [out]-> 32bit Adresse fuer Lesen
// [out]-> 32bit Groesse des Blocks
// [out]-> 32bit CRC des Blocks

#define IV_FUNCTION_END_GET_SEGMENT              0x0027
// -> [UINT16(SS_...)]
// <- [UINT16(SS_...)]
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_GET_SEGMENT und _END_GET_SEGMENT
#define IV_VALUE_SEGMENT_GET_UPDATE         0 // Es werden Daten gelesen ohne Bedienstop.
#define IV_VALUE_SEGMENT_GET_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_GET_SETUP          5
#define IV_VALUE_SEGMENT_GET_SETUP_EXTENDED 7

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfügung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_GET_SEGMENT_OK           0x0028
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

#define IV_FUNCTION_SYSTEM_RESET_OK              0x0029

// Transparentmodus aktivieren/deaktivieren.
#define IV_FUNCTION_OPEN_COM_SERVER_MODE         0x002A
#define IV_FUNCTION_CLOSE_COM_SERVER_MODE        0x002B

// Abfrage von Information zu einem Modultyp
#define IV_FUNCTION_GET_INFO                     0x002C
// param1: ACCESS.MODULE
// param2: bei ACCESS.MODULE UCHAR(BusNr) UCHAR(0=TM,1=L)

// Antwort in Com-Server-Mode gewechselt
#define IV_FUNCTION_OPEN_COM_SERVER_MODE_OK      0x002D

// Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
#define IV_FUNCTION_SET_NOTIFY                   0x002E
// param1: Typ der Benachrichtigung
// param2: Zu setztender Zustand der Benachrichtigung
#define IV_FUNCTION_GET_NOTIFY                   0x002F
// param1: Typ der Benachrichtigung
// Werte siehe IV_MODULE_APPLICATION.IV_FUNCTION_NOTIFY

#define IV_FUNCTION_GET_SOFTWARE_INFO            0x0030

// Lesen einer Seite des Flash (Bereich: 0 - 4095, Seite hat 528Byte)
#define IV_FUNCTION_GET_FLASH_PAGE               0x0031

// Setzen/Lesen der Systemflags von EEPROM-Adresse: 0x0ECC
// Bit1: Vorhandenes Setup und Listen werden übernommen und durch das Update nicht gelöscht.
//       Nur Mobil-MasterIV. Die Mindestversion auf dem Gerät muß 04.01.04.32 sein und die aufzuspielende mindestens 04.01.04.59 damit diese Option korrekt unterstützt wird.;
// Bit2-32: Reserved
#define IV_FUNCTION_SET_SYSTEM_FLAGS             0x0032
#define IV_FUNCTION_GET_SYSTEM_FLAGS             0x0033

// Schreiben/Lesen von: RAM
#define IV_FUNCTION_SET_RAM                      0x0034
#define IV_FUNCTION_GET_RAM                      0x0035

// Schreiben/Lesen von MMC
#define IV_FUNCTION_SET_MMC                      0x0036
#define IV_FUNCTION_GET_MMC                      0x0037
#define IV_FUNCTION_GET_SFSMPB                   0x0038

#pragma  pack(1)
// Strucktur fuer dynamische Verwaltung des SFS (MS) 06.11.08
typedef struct struct_memory_partition_block {

    unsigned int  StorageStartAddress; ///< Startadresse des Filessystems im Speicher
    unsigned int  StartFATAddress;     ///< Startadresse FAT
    unsigned int  DirStartAddress;     ///< Startadresse File Directory
    unsigned int  FileStartAddress;    ///< Startadresse Datenbereich
    unsigned int  FirstRecordAddress;  ///< Startadresse des Ringspeichers
    unsigned int  StorageEndAddress;   ///< Letzte Adresse des Speicherbereiches

    unsigned short  DirStartCluster;     ///< Start Cluster in dem das Dateiverzeichnis liegt
    unsigned short  FileStartCluster;    ///< Start Cluster für die Daten der Dateien
    unsigned short  FATClusterCount;     ///< Anzahl der Cluster die von der FAT belegt werden
    unsigned short  DirClusterCount;     ///< Anzahl der Cluster die vom Verzeichnis belegt werden.
    unsigned short  FileClusterCount;    ///< Anzahl der Cluster für den Dateibereich
    unsigned short  TotalClusterCount;   ///< Anzahl aller Cluster des Dateisystems
    unsigned short  SectorsPerCluster;   ///< Anzahl der Sektoren pro Cluster

    unsigned int  ClusterSize;         ///< Größe eines Clusters in Byte
    unsigned int  RecordVolumeSize;    ///< Größe des Ringspeichers in Byte
    unsigned int  ListVolumeSize;      ///< Größe des Dateisystem in Byte
    unsigned int  StorageVolumeSize;   ///< Größe der Memory-Card in Byte

    unsigned char ClusterShiftBit;     ///< Schiebebit für Clusteradressierung

} SFSMPB;
#pragma pack()

// MatchPort in den Monitor Mode (z-Mode mit aktivem WLAN Interface) wechseln
#define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT		 0x0039
// Antwort auf den Wechsel-Befehl BetriebsMode->MonitorMode
#define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK   0x003A

// System Device Log
#define IV_FUNCTION_GET_SYS_RECORD				 0x003B
#define IV_FUNCTION_SET_SYS_RECORD				 0x003C
#define IV_FUNCTION_RESTORE_ALLSYS_RECORDS		 0x003D

#define IV_ANSW_NO_SYS_RECORD               0xC0
#define IV_ANSW_ONE_SYS_RECORD				0xC1
#define IV_ANSW_ACK_ON_SYS_RECORD			0xC8
#define IV_SYS_RECORDS_MAX					0x200

// Passwort fuer Kommunikation
#define IV_FUNCTION_PASSWORD_CHECK              0x003E
#define IV_FUNCTION_PASSWORD_KEY                0x003F

#define IV_FUNCTION_PRESS_VIRTUAL_KEY           0x0040

// Schreiben eines Datenstroms
#define IV_FUNCTION_TRANSMIT_STREAM             0x0041
#define IV_FUNCTION_TRANSMIT_STREAM_OK          0x0042

#define IV_FUNCTION_RECEIVE_STREAM              0x0043

enum streamType { 
    update = 0, 
    update_bg,
    firmware,
    fonts,
    graphics,
    setup,
    setup_list,
    setup_extended,
    list_access,
    list_timeboy,
    fingerprint,
    text,
    touchkeys,
    displaycolors,
    wlanconfigs,
    fake
};

// Passwort fuer Systemkonfigurationen
#define IV_FUNCTION_SYSTEM_KEY_CHECK          0x0044
#define IV_FUNCTION_SYSTEM_KEY                0x0045

// Lesen der Flashdaten über die physikalische Adresse
#define IV_FUNCTION_GET_FLASH_PHYSICAL        0x0046

// Befehl zum Transparenten durchsenden von Daten.
// Hierbei werden die MasterIV-spezifischen Zeichen wie '~' maskiert gesendet,
// der Empfänger demaskiert diese je nach Param2-Flag, ebenso bei dem Empfang.
#define IV_FUNCTION_TRANSPARENCY_MODE         0x0047
// Param1: unsigned short Slot-Nummer oder id zu der durchgeschaltet werden soll.
// 1 - 254 -> Slotnummer
// 255 - .. -> Andere Bedeutung. Daten zur Id aus der bios_settings.
// Param2: unsigned short Flags
// Bit1: Wenn gesetzt (Raw-Modus) werden alle Zeichen 1:1 durchgeleitet. Ansonsten (Masked-Modus) werden die empfangenen bzw. zu sendenden Zeichen de-/maskiert.
// Bit2: Wenn gesetzt wird die Zeitangabe in Param5 in Sekunden gewertet, ansonsten in Millisekunden.
// Bit3: Wenn gesetzt sollen nur Timeboy I-III Geräte angesprochen werden. Da TimeboyIV Geräte einen TimeboyIII in ihrer Kommunikation nachahmen können
//       wird dieses Flag dazu verwendet, Bereits beim Herstellen des TransparentMode ältere Timeboystände zu verlangen. Nutzung im TbSetup und TbCom.dll
// Bit4-16: Unbenutzt
// Param3: unsigned int gewünschte Lesepuffergröße der Gegenstelle in Byte.
// Param4: unsigned int gewünschte Schreibpuffergröße der Gegenstelle in Byte.
// Param5: unsigned short Timeout bei Kommunikation über Raw-Modus in Milisekunden oder Sekunden, je nach Flag-Bit2.

// Befehle zum Schalten der gelben LED bei Dockingstation-V2
#define IV_FUNCTION_SET_LED                   0x0048
#define IV_FUNCTION_GET_LED                   0x0049
// Param1: unsigned char slot. Welcher Slot angesprochen werden soll.
// 0 -> Das Gerät mit der deviceId selbst.
// 1 - 253 -> Slotnummer
// Param2: unsigned char Led-Id welche LED angesprochen wird.
// Bei der Mehrfachdocking entspricht diese Id folgenden LED's.
// 1 -> Rote
// 2 -> Gelbe
// 3 -> Grüne
// Param3: unsigned char Status der Blinkfrequenz.
// Hierbei handelt es sich um die Prozentangabe der Zeit relativ zu einer Sekunde.
// 6,66% sind 66,6ms an, 933,4ms aus.
// 93,33% sind 933,3ms an, 66,7ms aus.
#define IV_VALUE_LED_OFF             ( 0x00 )    // Flag LedOff
#define IV_VALUE_LED_PERCENT01       ( 0x01 )    // Flag LedOn6,66%
#define IV_VALUE_LED_PERCENT02       ( 0x02 )    // Flag LedOn13,33%
#define IV_VALUE_LED_PERCENT03       ( 0x03 )    // Flag LedOn19,99%
#define IV_VALUE_LED_PERCENT04       ( 0x04 )    // Flag LedOn26,66%
#define IV_VALUE_LED_PERCENT05       ( 0x05 )    // Flag LedOn33,33%
#define IV_VALUE_LED_PERCENT06       ( 0x06 )    // Flag LedOn39,99%
#define IV_VALUE_LED_PERCENT07       ( 0x07 )    // Flag LedOn46,66%
#define IV_VALUE_LED_PERCENT08       ( 0x08 )    // Flag LedOn53,33%
#define IV_VALUE_LED_PERCENT09       ( 0x09 )    // Flag LedOn59,99%
#define IV_VALUE_LED_PERCENT10       ( 0x0A )    // Flag LedOn66,66%
#define IV_VALUE_LED_PERCENT11       ( 0x0B )    // Flag LedOn73,33%
#define IV_VALUE_LED_PERCENT12       ( 0x0C )    // Flag LedOn79,99%
#define IV_VALUE_LED_PERCENT13       ( 0x0D )    // Flag LedOn86,66%
#define IV_VALUE_LED_PERCENT14       ( 0x0E )    // Flag LedOn93,33%
#define IV_VALUE_LED_ON              ( 0x0F )    // Flag LedOn
// Bei setLED wird Param1 + Param2 + Param3 gesendet und als Antwort keiner erwartet.
// Bei getLED wird Param1 + Param2 gesendet und als Antwort Param1 + Param2 + Param3 erwartet.

class Channel4ModuleSystem : public Channel4ModuleHardware
{
public:
    Channel4ModuleSystem( short channelId = IV_DEFAULT_CHANNELID, short deviceId = IV_DEFAULT_DEVICEID );
    virtual ~Channel4ModuleSystem();

public:
    unsigned int _serial; // Seriennummer des Geraetes

    inline void setPasswordCommunication( char *password )
    {
        memset( _passwordCommunication, 0, sizeof(_passwordCommunication) );
        strncpy( _passwordCommunication, password, sizeof(_passwordCommunication) - 1 );
    }
    inline const char *getPasswordCommunication()
    {
        return _passwordCommunication;
    }
    inline void setFlagPasswordProtected( int bState )
    {
        _isPasswordProtected = (bState == 0) ? 0 : 1;
    }
    // Ist Passwortschutz aktiv?
    inline int isPasswordProtected()
    {
        return _isPasswordProtected;
    }
    inline void setFlagPasswordProtection( int bState )
    {
        _isPasswordProtection = (bState == 0) ? 0 : 1;
    }
    // Wird Passwortschutz unterstützt?
    inline int isPasswordProtection()
    {
        return _isPasswordProtection;
    }

private:
    char _passwordCommunication[17]; // Passwort fuer die Kommunikation
    int _isPasswordProtected; // Haelt die Information, dass das Geraet Passwortgeschuetzt ist.
    int _isPasswordProtection; // Haelt die Information, dass das Geraet den Passwortschutz unterstuetzt.

public:
    char _versionFirmware[64];

public:
    int readData( void *buffer, int readCount, int &readInCount, int readTrials = IV_READ_TRIALS );
    int writeData( const void *buffer, int writeCount, int &writeOutCount, int writeTrials = IV_WRITE_TRIALS );
    enum EnumTraceDataType
    {
        TraceDataWrite,
        TraceDataRead
    };
    enum EnumTraceDataMode
    {
        TraceDataFull,
        TraceDataFullWithCrLf,
        TraceDataBegin,
        TraceDataAppend,
        TraceDataEnd,
        TraceDataEndWithCrLf
    };
    void traceData( unsigned char *data, int size, EnumTraceDataMode mode = TraceDataFullWithCrLf, const char *prefix = nullptr );

public:
    // Gibt Modulfehler zurueck.
    virtual void getErrorModule( short module, short error, std::string &text );
    // Gibt Funktionsfehlertext zurueck.
    virtual void getErrorFunction( short module, short func, short error, std::string &text );

    // MODULE_HARDWARE
    // BUG:
    // Im Bootloader 00.04.00.00 && 00.04.00.01 liefert die Funktion
    // - TypeTransponder den Wert fuer das Display
    // - TypeDisplay den Wert fuer das Keypad
    // - TypeKeypad den Wert fuer den Transponder
    int setTypeDisplay( short deviceId, unsigned char val, unsigned short value2 = 0xFFFF, int checked = FALSE );
    int getTypeDisplay( short deviceId, unsigned char &val, int checked = FALSE );
    int setTypeKeypad( short deviceId, unsigned char val, unsigned short value2 = 0xFFFF, int checked = FALSE );
    int getTypeKeypad( short deviceId, unsigned char &val, int checked = FALSE );
    int setTypeTransponder( short deviceId, unsigned char val, unsigned short value2 = 0xFFFF, int checked = FALSE );
    int getTypeTransponder( short deviceId, unsigned char &val, int checked = FALSE );

    // Gesamte Hardwareinfo lesen
    int getHardwareInfo( short deviceId, StructHardwareInfo &hwInfo );
    // Gesamte Hardwareinfo in String umsetzen.
    int getInfos( StructHardwareInfo hwInfo, string &info );
    
    // MODULE_SYSTEM

    // Systemreset; Speicherbereiche loeschen
    int systemReset( short deviceId, unsigned short modus );
    int systemResetOK( short deviceId );

    // Firmwareversion lesen
    int getVersion( short deviceId, char *version );
    int getVersionBootloader( short deviceId, char *version );

    // Seriennummer lesen
    int setSerial( short deviceId, unsigned int serial, unsigned int value2 = 0xFFFFFFFF );
    virtual int getSerial( short deviceId, unsigned int &serial );

    // System Device Log
    int systemReadRecord( short deviceId, unsigned char *buf, int &count, unsigned char &status );
    int systemQuitRecord( short deviceId, unsigned char &status );
    int systemRestoreRecords( short deviceId );

    // Datum/Uhrzeit setzen/lesen
    int setRTC( short deviceId, char *time );
    int getRTC( short deviceId, char *time );

    // Helper für Systemvariablen von Mobile
    void mobileMaskedSysvar( char *buf );
    void mobileUnmaskedSysvar( char *buf );
    unsigned char hexToUINT8( char *data );
    // Systemvariable setzen/lesen
    int setSystemVariable( short deviceId, char *name, char *val );
    int getSystemVariable( short deviceId, const char *name, char *val );

    // Relais setzen/lesen
    int setRelay( short deviceId, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time );
    int getRelay( short deviceId, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time );

protected:
    unsigned int m_crcSetSegment, m_crcGetSegment;
    std::string m_streamBuffer;
    bool m_streamTransfer;
public:
    void setStreamTransfer( bool state )
    { 
        m_streamTransfer = state;
    }
    bool isStreamTransfer()
    { 
        return m_streamTransfer;
    }
    void clearStreamTransfer()
    { 
        m_streamBuffer.clear();
    }
public:
    // Datensegmentuebertragung beginnen/enden
    int beginSetSegment( short deviceId, unsigned short art, unsigned int &address, unsigned int size, unsigned int crc );
    int beginSetSegmentOK( short deviceId );
    int endSetSegment( short deviceId, unsigned short art, unsigned int crc );
    int endSetSegmentOK( short deviceId );

    // Datensegmentuebertragung beginnen/enden
    int beginGetSegment( short deviceId, unsigned short art, unsigned int &address, unsigned int &size, unsigned int &crc );
    int beginGetSegmentOK( short deviceId );
    int endGetSegment( short deviceId, unsigned short art );
    int endGetSegmentOK( short deviceId );

    // Flash setzen/lesen mittels logischer Adresse (die ersten 16kByte Bootloader koennen nur gelesen werden)
    int setFlash( short deviceId, unsigned int address, unsigned char *buf, int count );
    int getFlash( short deviceId, unsigned int address, unsigned char *buf, int count );
    int getFlashPage( short deviceId, unsigned short pageNum, unsigned short offs, unsigned char *buf, int &count );
    // Flashdaten lesen, mittels physikalischer Adresse.
    int getFlashPhysical( short deviceId, unsigned int address, unsigned char *buf, int count );

    // EEPROM setzen/lesen
    int setEEPROM( short deviceId, unsigned int address, unsigned char *buf, int count );
    int getEEPROM( short deviceId, unsigned int address, unsigned char *buf, int count );

    // PROGRAM setzen/(lesen unterbunden)
    int setProgram( short deviceId, unsigned int address, unsigned char *buf, int count );
    int getProgram( short deviceId, unsigned int address, unsigned char *buf, int count );

    // RAM setzen/lesen
    int setRAM( short deviceId, unsigned int address, unsigned char *buf, int count );
    int getRAM( short deviceId, unsigned int address, unsigned char *buf, int count );

    // MMC schreiben/lesen
    int setMMC( short deviceId, unsigned int address, unsigned char *buf, int count );
    int getMMC( short deviceId, unsigned int address, unsigned char *buf, int count );

    // Info zum Dateisystem
    int getSFSMPB( short deviceId, unsigned char &type, SFSMPB &sfs );

    // Systembios aufrufen/verlassen
    int enterBios( short deviceId );
    int enteredBios( short deviceId );
    int exitBios( short deviceId );
    int exitedBios( short deviceId );

    // Interface setzen/lesen (RS232, TCPIP...)
    int setInterface( short deviceId, unsigned char type);
    int getInterface( short deviceId, unsigned char &type);
    
    // Baud-Rate setzen/lesen
    int setBaud( short deviceId, unsigned int br);
    int getBaudRate( short deviceId, unsigned int &br);

    // DeviceID setzen/lesen
    int setDeviceIdBios( short deviceId, unsigned short id);
    int getDeviceIdBios( short deviceId, unsigned short &id);

    // Bootloader aktivieren
    int enterBootloader( short deviceId);

    // Monitor Mode vom XPort
    int enterXPortMonitor( short deviceId, char *data, int &count );
    int enterXPortMonitorOK( short deviceId );
    int enterXPortMonitorExt( short deviceIDRemote, char *data, int &count );
    int enterXPortMonitorExtOK( short deviceId );
    int commandXPort( short deviceId, char *buf, int &count );
    int exitXPortMonitor( short deviceId );
    int exitXPortMonitorOK( short deviceId );

    // Transparentsmodus fuer COM
    int openComServerMode( short deviceId, char *comm, unsigned int baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose );
    int openComServerModeOK( short deviceId );
    int closeComServerMode( short deviceId );

    // Informationen zu einem Modultyp abfragen
    int getInfo( short deviceId, const char *name, const char *param, char *info, int &infoLength );

    // Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
    int setNotify( short deviceId, unsigned char type, unsigned char state );
    int getNotify( short deviceId, unsigned char type, unsigned char &state );

    // Softwareinformation der Anwendungsfirmware abfragen
    int getSoftwareInfo( short deviceId, unsigned char *swi, int &infoLength );

    // Setzen/Lesen der Systemflags
    int setSystemFlags( short deviceId, unsigned int flags );
    int getSystemFlags( short deviceId, unsigned int &flags );
    
    // Passwort pruefen
    int passwordCheck( short deviceId, char *password );
    // Passwortschlüssel abrufen
    int passwordKey( short deviceId, char *key );

    // Einen Tastencode ausführen. F1-F15 drücken.
    int pressVirtualKey( short deviceId, unsigned char vk, unsigned char flags );

    // Daten als ein Datenstrom senden.
    int writeStream( short deviceId, streamType type, const char *buf, int count );
    // Daten als ein Datenstrom empfangen.
    int readStream( short deviceId, streamType type, char *buf, int count );

    // Systemschlüssel pruefen
    int systemKeyCheck( short deviceId, char *key );
    // Systemschlüssel abrufen
    int systemKey( short deviceId, char *key );

private:
    enum EnumTranparencyMode
    {
        Off,    // Nicht aktiv.
        Masked, // Aktiv, wobei Sonderzeichen maskiert werden.
        Raw     // Aktiv, alles wird 1:1 weitergeleitet.
    };
    EnumTranparencyMode _transparencyMode;
public:
    // Daten transparent übertragen.
    int startTransparencyMode( unsigned char deviceId, unsigned short id, unsigned short flags, unsigned int sizeReadBuffer, unsigned int sizeWriteBuffer, int timeout );
    int stopTransparencyMode( unsigned char deviceId );
    bool isTransparencyMode();

    // Gelbe LED bei Dockingstation-V2 setzen/lesen
    int setLEDState( short deviceId, unsigned char slot, unsigned char id, unsigned char state);
    int getLEDState( short deviceId, unsigned char slot, unsigned char id, unsigned char &state);

public:
    // Ausfuehren eines Protokollbefehls
    virtual int call( short context, short module, short function, unsigned char *buf, int &count, int contextValue = -1 );

    // Kommunikations-Worker anhalten.
    virtual int suspend()
    {
        return IV_ERROR_SUCCESS;
    }

    // Kommunikations-Worker weiterlaufen lassen.
    virtual void resume()
    {
    }

    //#################################
    // remoteCall - Routinen

    // Ausfuehren von Befehlen
    virtual int remoteCall( short module, short func, unsigned char *data, int &count );

};

#endif
