/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "Channel4ModuleTable.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
Channel4ModuleTable::Channel4ModuleTable( short channelID, short deviceID )
    : Channel4ModuleApplication( channelID, deviceID ) 
{
    trace( "%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, channelID, deviceID );
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
Channel4ModuleTable::~Channel4ModuleTable() 
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void Channel4ModuleTable::getErrorModule( short module, short error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_TABLE ?
    if ( module == IV_MODULE_TABLE )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ModuleApplication::getErrorModule( module, error, text );
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void Channel4ModuleTable::getErrorFunction( short module, short func, short error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_TABLE )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 )
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %i ): Korrektes Passwort für die Kommunikation erforderlich." ) % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %i ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt." ) % error );
            return;
        }

        switch ( func )
        {
        case IV_FUNCTION_GET_TABLE:
        case IV_FUNCTION_GET_FIELD:
            switch ( error )
            {
            case 1: text = boost::str( boost::format( "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
            case 2: text = boost::str( boost::format( "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
            case 3: text = boost::str( boost::format( "( %i ): Der Subminorbestandteil (Feldname) des Tabellenfeldnamens ist unbekannt." ) % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;

        case IV_FUNCTION_TABLE_OPEN:
            switch ( error )
            {
            case 1: text = boost::str( boost::format( "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
            case 2: text = boost::str( boost::format( "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
                // Standardmaessig ist die Option mit 0 angegeben. Ein gesetztes Bit, dessen Bedeutung unklar ist, ergibt diesen Fehler.
            case 3: text = boost::str( boost::format( "( %i ): Unbekannte Option gesetzt." ) % error ); break;
                // Wenn ein reservierter Parameter angegeben wurde.
            case 4: text = boost::str( boost::format( "( %i ): Der angegebene Reservierte Parameter wird nicht unterstützt." ) % error ); break;
                // Die Tabelle ist bereits mit schreib - Recht geoffnet worden.
            //case 5: text = boost::str( boost::format("( %i ): Die Tabelle kann nur lesend geoeffnet werden." ) % error ); break;
                // Die Tabelle ist bereits mit Exclusive - Recht geoffnet worden.
            //case 6: text = boost::str( boost::format("( %i ): Die Tabelle ist z. Z. gesperrt." ) % error ); break;
                // Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt. (Zu oft geoeffnet worden usw..)
            //case 7: text = boost::str( boost::format("( %i ): Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt." ) % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;

        case IV_FUNCTION_TABLE_CLOSE:
            switch ( error )
            {
                // Wenn versucht wird ein Handle zu schliessen der keiner offenen Tabelle zugehoerig ist.
            case 1: text = boost::str( boost::format( "( %i ): Angegebener Handle war nicht geoffnet." ) % error ); break;
                // Liste konnte nicht geschlossen werden.
            case 2: text = boost::str( boost::format( "( %i ): Tabelle konnte nicht geschlossen werden." ) % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;

        case IV_FUNCTION_TABLE_SET_FILTER:
        case IV_FUNCTION_TABLE_SET_FILTER_OK:
            switch ( error )
            {
                // Wenn der angegebene Handle nicht existiert.
            case 1: text = boost::str( boost::format( "( %i ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                // Ungueltiger Selektionsspaltenanzahl.
            case 2: text = boost::str( boost::format( "( %i ): Falsche Anzahl von Selektionsspalten." ) % error ); break;
                // Nach setzen des Filterkriteriums -> Kein Datensatz
            case 3: text = boost::str( boost::format( "( %i ): Filterkriterium liefert kein Ergebnis." ) % error ); break;
                // Spaltenname des Filter unbekannt.
            case 4: text = boost::str( boost::format( "( %i ): Unbekannter Spaltenname." ) % error ); break;
                // Unbekannter (nicht unterstuetzter) Filteroperator.
            case 5: text = boost::str( boost::format( "( %i ): Unbekannter Filteroperator." ) % error ); break;
                // Fehlender in Anfuehrungszeichenpaar des zu filterndem Spaltenwerts.
                // Keine vorhanden, unterschiedliches ein/aus '" oder fehlender ein oder Ausgang.
            case 6: text = boost::str( boost::format( "( %i ): Fehler in Anfuehrungszeichenpaar des zu filterndem Spaltenwertes." ) % error ); break;
                // Fehler im zu Filterndem Spaltenwert. z. B. Buchstaben obwohl nur Ziffern, zu lange Angabe etc.
            //case 7: text = boost::str( boost::format("( %i ): Fehler im zu filterndem Spaltenwert." ) % error ); break;
                // Syntaxfehler im Filter. z. B. Abschluss des Filters mit Semikolon (reserviert fuer zukuenftige Erweiterung auf mehrer Spalten).
            //case 8: text = boost::str( boost::format("( %i ): Im zu setzendem Filter ist ein Syntaxfehler." ) % error ); break;

            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;
        case IV_FUNCTION_TABLE_GET_FILTER:
        case IV_FUNCTION_TABLE_CLEAR_FILTER:
            switch ( error )
            {
                // Wenn der angegebene Handle nicht existiert.
            case 1: text = boost::str( boost::format( "( %i ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                // Auf der angegebenen Tabelle ist kein Filter gesetzt.
            //case 2:text = boost::str( boost::format("( %i ): Kein Filter bei Tabelle angegeben." ) % error ); break;

            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;

        case IV_FUNCTION_TABLE_SKIP:
            switch ( error )
            {
                // Wenn der angegebene Handle nicht existiert.
            case 1: text = boost::str( boost::format( "( %i ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                // Wert fuer Origin ausserhalb des gueltigen Bereichs. (0, 1, 2)
            case 2: text = boost::str( boost::format( "( %i ): Wert fuer Origin ungueltig." ) % error ); break;
                // Sprungweite (Offset) fuehrte zu BOF (begin of file). Der Datensatzzeiger steht damit vor dem ersten Datensatz.
            case 3: text = boost::str( boost::format( "( %i ): Sprung fuehrte zu BOF." ) % error ); break;
                // Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
            case 4: text = boost::str( boost::format( "( %i ): Sprung fuehrte zu EOF." ) % error ); break;
                // Sprung konnte aufgrund von angegebenen offset mit origin nicht durchgefuehrt werden.
                // Z. B. Filter gesetzt und Sprung auf aktuellen der nicht dem Filterkriterium entspraeche.
            case 5: text = boost::str( boost::format( "( %i ): Sprung konnte nicht durchgefuehrt werden." ) % error ); break;

            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;
        case IV_FUNCTION_TABLE_SET_FIELD_VALUE:
            switch ( error )
            {
                // Wenn der angegebene Handle nicht existiert.
            case 1: text = boost::str( boost::format( "( %i ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                // Spaltenname wurde nicht gefunden.
            case 2: text = boost::str( boost::format( "( %i ): Spaltenname unbekannt." ) % error ); break;
                // Angegebener Wert ist zu lang
            case 3: text = boost::str( boost::format( "( %i ): Zu setzender Wert ist zu lang." ) % error ); break;
                // Formatfehler in zu setzendem Spaltenwert.
            case 4: text = boost::str( boost::format( "( %i ): Formatfehler in zu setzendem Spaltenwert." ) % error ); break;
                // Schreiben des Spaltenwerts fehlgeschlagen
            case 5: text = boost::str( boost::format( "( %i ): Schreiben des Spaltenwerts fehlgeschalgen." ) % error ); break;

            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;
        case IV_FUNCTION_TABLE_GET_FIELD_VALUE:
            switch ( error )
            {
                // Wenn der angegebene Handle nicht existiert.
            case 1: text = boost::str( boost::format( "( %i ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                // Spaltenname wurde nicht gefunden.
            case 2: text = boost::str( boost::format( "( %i ): Spaltenname unbekannt." ) % error ); break;

            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;
        case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
        case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK:
        case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
        case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK:
        case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
        case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK:
            switch ( error )
            {
            case 1: text = boost::str( boost::format( "( %i ): Das gewählte Templateformat ist unbekannt." ) % error ); break;
            case 2: text = boost::str( boost::format( "( %i ): Das Modul antwortet nicht auf die Anfrage." ) % error ); break;
            case 3: text = boost::str( boost::format( "( %i ): Das gewählte Templateformat entspricht nicht dem verwendeten." ) % error ); break;
            case 7: text = boost::str( boost::format( "( %i ): Das Modul kann zur Zeit nicht angesprochen werden." ) % error ); break;
            case 129: text = boost::str( boost::format( "( %i ): Das Modul bringt einen nicht näher spezifizierten Fehler." ) % error ); break;
            case 132: text = boost::str( boost::format( "( %i ): Die Datenbank des Moduls ist voll." ) % error ); break;
            case 133: text = boost::str( boost::format( "( %i ): Das Modul enthält keine Fingertemplates." ) % error ); break;
            case 134: text = boost::str( boost::format( "( %i ): Das Fingertemplate existiert bereits." ) % error ); break;
            case 135: text = boost::str( boost::format( "( %i ): Das angeforderte Fingertemplate existiert nicht." ) % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;
        case IV_FUNCTION_TABLE_GENERATE:
        case IV_FUCNTION_TABLE_GENERATE_OK:
            switch( error )
            {
            case 7: text = boost::str( boost::format( "( %i ): Das Modul kann zur Zeit nicht angesprochen werden." ) % error ); break;
            case 129: text = boost::str( boost::format( "( %i ): Das Modul bringt einen nicht näher spezifizierten Fehler." ) % error ); break;
            case 133: text = boost::str( boost::format( "( %i ): Das Modul enthält keine Daten zum erzeugen einer Liste." ) % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
            }
            break;

        default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Fehlerblock für Funktion!" ) % error ); break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ModuleApplication::getErrorFunction( module, func, error, text );
}

//#########################################################
//####################### TABLE ###########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::remoteCall( short module, short func, unsigned char *data, int &count )
{
    if ( module == IV_MODULE_TABLE )
    {
        switch ( func )
        {
        // Ermitteln der Tabellendefinition
        case IV_FUNCTION_GET_TABLE: 
            return getTable( getDeviceId(), reinterpret_cast<char *>( data ), reinterpret_cast<StructTable2 &>( data[strlen(reinterpret_cast<char *>( data )) + 1] ) );

        // Ermitteln der Felddefinition
        case IV_FUNCTION_GET_FIELD: 
            return getField( getDeviceId(), reinterpret_cast<char *>( data ), reinterpret_cast<StructTable2Field &>( data[strlen( reinterpret_cast<char *>( data )) + 1] ) ); 

        case IV_FUNCTION_TABLE_OPEN:
        case IV_FUNCTION_TABLE_CLOSE:
        case IV_FUNCTION_TABLE_SET_FILTER:
        case IV_FUNCTION_TABLE_SET_FILTER_OK:
        case IV_FUNCTION_TABLE_GET_FILTER:
        case IV_FUNCTION_TABLE_CLEAR_FILTER:
        case IV_FUNCTION_TABLE_SKIP:
        case IV_FUNCTION_TABLE_SET_FIELD_VALUE:
        case IV_FUNCTION_TABLE_GET_FIELD_VALUE:
        case IV_FUNCTION_TABLE_SET_RECORD:
        case IV_FUNCTION_TABLE_GET_RECORD:
        case IV_FUNCTION_TABLE_GENERATE:
            // Funktion nicht definiert
            return 0x0200; 

        case IV_FUCNTION_TABLE_GENERATE_OK:
            return tableGenerateOk( getDeviceId() );

        case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
            // Funktion nicht definiert
            return 0x0200; 

        case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK:
            return fingerprintTableAppendRecordOk( getDeviceId() );

        case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
            // Funktion nicht definiert
            return 0x0200; 

        case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK:
            return fingerprintTableGetRecordOk( getDeviceId() );

        case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
            // Funktion nicht definiert
            return 0x0200; 

        case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK:
            return fingerprintTableDeleteRecordOk( getDeviceId() );

        case IV_FUNCTION_TABLE_GET_RECORD_COUNT:
        case IV_FUNCTION_TABLE_GET_RECORD_NUM:
        // Funktion nicht definiert
        default: return 0x0200; 
        }

    }

    return Channel4ModuleApplication::remoteCall( module, func, data, count );
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::getTable( short deviceID, char *name, StructTable2 &table )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, STable2 ]\n", __FILE__, __LINE__, __func__, deviceID, name );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );

        // Name als Parameter eintragen
        count = sprintf( reinterpret_cast<char *>( _packetData ), "%s", name ) + 1;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE );
            return _result;
        }

        // OK
        if ( count != sizeof( StructTable2 ) )
        {
            trace( "%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Struktur uebernehmen
        table = *reinterpret_cast<StructTable2 *>( _packetData );
    }
    else
    {
        unsigned char data[256];
        StructTable2 t;
        count = 0;
        short err = IV_ERROR_SUCCESS;
        
        strcpy( reinterpret_cast<char *>( data ), name );
        memset( &t, 0, sizeof( StructTable2 ) );

        char *major;
        char *minor;

        major = strtok( reinterpret_cast<char *>( data ), "." );
        minor = strtok( NULL, "." );

        if ( strcmp( major, "RECORD" ) == 0 )
        {
            switch ( atoi( minor ) )
            {
            case 0:
                t.num = 0;
                strcpy( t.name, "PZE-Meldung" );
                t.count = 5;
                break;

            case 1:
                t.num = 1;
                strcpy( t.name, "Status" );
                t.count = 8;
                break;

            default:
                // Minorbestandteil undefiniert
                err = 0x0002;
            }
        }
        else if ( strstr( reinterpret_cast<char *>( data ), "LIST" ) == reinterpret_cast<char *>( data ) )
        {
                // Minorbestandteil undefiniert
                err = 0x0002;
        }
        else if (strstr(reinterpret_cast<char *>( data ), "ACCESS") == reinterpret_cast<char *>( data ) )
        {
                // Minorbestandteil undefiniert
                err = 0x0002;
        }
        else
        {			
            // Majorbestandteil undefiniert.
            err = 0x0001;
        }

        if (err == IV_ERROR_SUCCESS)
        {
            memcpy( &data[count], &t, sizeof( StructTable2 ) );
            count = sizeof( StructTable2 );
        }
        else
        {
            *reinterpret_cast<short *>( data ) = err;
            count = sizeof( short );
        }
        // Antwort des Befehls senden.
        if ( (_result = call( (err == IV_ERROR_SUCCESS) ? IV_CONTEXT_TRANSMIT_ANSWER : IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE, data, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::getField( short deviceID, char *name, StructTable2Field &field )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, StructTable2Field ]\n", __FILE__, __LINE__, __func__, deviceID, name );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        count = sprintf( reinterpret_cast<char *>( _packetData ), "%s", name ) + 1;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD );
            return _result;
        }

        // OK
        if ( count != sizeof( StructTable2Field ) )
        {
            trace( "%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Sturktur uebernehmen
        field = *reinterpret_cast<StructTable2Field *>( _packetData );
    }
    else
    {
        unsigned char data[256];
        StructTable2Field f;
        count = 0;
        short err = IV_ERROR_SUCCESS;

        strcpy( reinterpret_cast<char *>( data ), name );
        memset( &field, 0, sizeof( field ) );
        char *major;
        char *minor;
        char *subminor;

        major = strtok( reinterpret_cast<char *>( data ), "." );
        minor = strtok( NULL, "." );
        subminor = strtok( NULL, "." );

        if ( strcmp( major, "RECORD" ) == 0 )
        {
            switch ( atoi( minor ) )
            {
            case 0:
                {
                switch ( atoi( subminor ) )
                {
                case 0:
                    f.num = 0;
                    strcpy( f.name, "DU" );
                    f.type = TF_DT;
                    f.size = 7;
                    break;

                case 1:
                    f.num = 1;
                    strcpy( f.name, "Kennung" );
                    f.type = TF_A_NUM;
                    f.size = 5;
                    break;

                case 2:
                    f.num = 2;
                    strcpy( f.name, "PID" );
                    f.type = TF_A_NUM;
                    f.size = 9;
                    break;

                case 3:
                    f.num = 3;
                    strcpy( f.name, "IP" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                case 4:
                    f.num = 4;
                    strcpy( f.name, "Port" );
                    f.type = TF_A_NUM;
                    f.size = 9;
                    break;

                default:
                    err = 0x0003;
                }
                break;
                }
            case 1: 
                {
                switch ( atoi( subminor ) )
                {
                case 0:
                    f.num = 0;
                    strcpy( f.name, "DU" );
                    f.type = TF_DT;
                    f.size = 7;
                    break;

                case 1:
                    f.num = 1;
                    strcpy( f.name, "Feld1" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                case 2:
                    f.num = 2;
                    strcpy( f.name, "Feld2" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                case 3:
                    f.num = 3;
                    strcpy( f.name, "Feld3" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                case 4:
                    f.num = 4;
                    strcpy( f.name, "Feld4" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                case 5:
                    f.num = 5;
                    strcpy( f.name, "Feld5" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;
                case 6:
                    f.num = 6;
                    strcpy( f.name, "Feld6" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                case 7:
                    f.num = 7;
                    strcpy( f.name, "Feld7" );
                    f.type = TF_A_NUM;
                    f.size = 21;
                    break;

                default:
                    err = 0x0003;
                }
                break;}
            default:
                err = 0x0002;
            }
        }
        else if ( strcmp( major, "LIST" ) == 0 )
        {
                err = 0x0002;
        }
        else if ( strcmp( reinterpret_cast<char *>( data ), "ACCESS") == 0 )
        {
                err = 0x0002;
        }
        else
        {			
            // Majorbestandteil undefiniert.
            err = 0x0001;
        }

        if ( err == IV_ERROR_SUCCESS )
        {			
            memcpy( &data[count], &f, sizeof( StructTable2Field ) );
            count = sizeof( StructTable2Field );
        }
        else
        {
            *reinterpret_cast<short *>( data ) = err;
            count = sizeof( short );
        }

        // Antwort des Befehls senden.
        if ( (_result = call( (err == IV_ERROR_SUCCESS) ? IV_CONTEXT_TRANSMIT_ANSWER : IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD, data, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableOpen( short deviceID, char *name, int options, int &handle, char *reserved )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, name, options, handle, reserved );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Parameter eintragen
        count = sprintf( reinterpret_cast<char *>( _packetData ), "%s", name ) + 1;

        *reinterpret_cast<int *>( &_packetData[count] ) = options;
        count += sizeof( int );

        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", reserved ) + 1;


        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN );
            return _result;
        }

        // OK
        // Ein leerbyte fuer reservierten String in Antwort
        if ( count != (sizeof( int ) + sizeof( unsigned char )) )
        {
            trace( "%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Handle uebernehmen
        handle = *reinterpret_cast<int *>( _packetData );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
        
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableClose( short deviceID, int handle )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, handle );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Parameter eintragen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof( int );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
        
    }

    return _result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableSetFilter( short deviceID, int handle, char *filter )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, filter );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Parameter eintragen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof( int );
        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", filter ) + 1;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace("%s(%i):WARNING(3): %s Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    /*
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ( (_result = Pending( deviceID, module, func, error, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            Trace( "%s(%i):ERROR: %s.Pending [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        if ( (module == IV_MODULE_TABLE) && (func == IV_FUNCTION_TABLE_SET_FILTER_OK) )
        {
            Trace( "%s(%i):INFO(3): %s.Pending [ Antwort OK ]\n", __FILE__, __LINE__, __func__ );
        }

        // evntl. aktiven Fehler auswerten
        if ( error != IV_ERROR_SUCCESS )
        {
            _result = error;
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK );
            return _result;
        }
    */
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, _packetData, count, IV_PENDING_TRIALS( getTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK );
            return _result;
        }

        // Keine Aktive - Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s.Pending [ Aktive - Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
        
    }


    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableSetFilterOk( short deviceID )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }


    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGetFilter( short deviceID, int handle, char *filter, short &len )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceID, filter, len );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );

        // Parameter uebernehmen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof(int);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER );
            return _result;
        }

        // OK
        if ( count > 80 )
        {
            trace( "%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Wert uebernehmen
        strcpy( filter, reinterpret_cast<char *>( _packetData ) );
        len = static_cast<short>( strlen( filter ) );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
        
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableClearFilter( short deviceID, int handle )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, handle );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof(int);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
        
    }

    return _result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableSkip( short deviceID, int handle, int offset, short origin )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, offset, origin );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof( int );
        *reinterpret_cast<int *>( &_packetData[count] ) = offset;
        count += sizeof( int );
        *reinterpret_cast<short *>( &_packetData[count] ) = origin;
        count += sizeof( short );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableSetField( short deviceID, int handle, char *name, char *val )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, name, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof( int );
        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", name ) + 1;
        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", val ) + 1;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGetField( short deviceID, int handle, char *name, char *val, short &len )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceID, name, val, len );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );

        // Name als Parameter eintragen
        *reinterpret_cast<int *>( _packetData ) = handle;
        count = sizeof( int );
        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", name ) + 1;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE );
            return _result;
        }

        // OK
        if ( count > 440 ) // Binaerfeld mit 220 Byte als Hexstring.
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Wert uebernehmen
        strcpy( val, reinterpret_cast<char *>( _packetData ) );
        len = static_cast<short>( strlen( val ) );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableSetRecord( short deviceID, int handle, unsigned char *data, short len )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceID, handle, data, len );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        *reinterpret_cast<int *>( &_packetData[count] ) = handle;
        count = sizeof( int );
        memcpy( &_packetData[count], data, len );
        count += len;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGetRecord( short deviceID, int handle, unsigned char *data, short &len )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceID, handle, data, len );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );

        // Name als Parameter eintragen
        *reinterpret_cast<int *>( &_packetData[count] ) = handle;
        count = sizeof(int);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD );
            return _result;
        }

        // OK
        if ( count > 256 )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        if ( count > len )
        {
            trace( "%s(%i):WARNING(1): %s [ Puffer für Antwortdaten zu klein gewählt. ]\n", __FILE__, __LINE__, __func__, count );
            // Wert uebernehmen
            memcpy( data, reinterpret_cast<char *>( _packetData ), len );
        }
        else
        {
            // Wert uebernehmen
            memcpy( data, reinterpret_cast<char *>( _packetData ), count );
            len = count;
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGetRecordCount( short deviceID, int handle, int &recordCount )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, recordCount );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        *reinterpret_cast<int *>( &_packetData[count] ) = handle;
        count = sizeof( int );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT );
            return _result;
        }

        // OK
        if ( count > static_cast<int>( sizeof( int ) ) )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Anzahl uebernehmen
        recordCount = *reinterpret_cast<int *>( _packetData );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGetRecordNum( short deviceID, int handle, int &recordNum )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, recordNum );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Name als Parameter eintragen
        *reinterpret_cast<int *>( &_packetData[count] ) = handle;
        count = sizeof( int );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM );
            return _result;
        }

        // OK
        if ( count > static_cast<int>( sizeof( int ) ) )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Anzahl uebernehmen
        recordNum = *reinterpret_cast<int *>( _packetData );
    }
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }	
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::fingerprintTableAppendRecord( short deviceID, unsigned char type, unsigned char *data, int size )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceID, type, data, size );

    int count = 0;
    int i;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );


        // Parameter eintragen
        _packetData[count] = type;
        count = sizeof( unsigned char );

        for ( i = 0; i < size; ++i )
        {
            _packetData[count + i] = data[i];
        }
        count += size;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD );
            return _result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, _packetData, count, IV_PENDING_FINGERPRINT_TRIALS( getTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK );
            return _result;
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::fingerprintTableAppendRecordOk( short deviceID )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::fingerprintTableGetRecord( short deviceID, unsigned char &type, unsigned int pid, unsigned char fid, unsigned char *data, int &size )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceID, type, pid, fid, data, size );

    int count = 0, i;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Parameter eintragen
        _packetData[count] = type;
        count = sizeof( unsigned char );
        *reinterpret_cast<unsigned int *>( &_packetData[count] ) = pid;
        count += sizeof( unsigned int );
        _packetData[count] = fid;
        count += sizeof( unsigned char );

        if ( (_result = call(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Wenn die Funktion unbekannt ist,
            // wird die Vorgaengerversion ohne "Aktive Antwort" geprueft.
            if ( ((_result >> 8) & 0xFF) == 2 )
            {
                if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1, _packetData, count )) != IV_ERROR_SUCCESS )
                {
                    // Fehler ausgeben
                    traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1 );
                    return _result;
                }

                // Templatetype uebernehmen
                type = _packetData[0];
                // Templatedaten uebernehmen
                size = count - 1;
                for ( i = 1; i < count; ++i )
                {				
                    data[i - 1] = _packetData[i];
                }
                count += size;

                // OK zurueckmelden
                _result = IV_ERROR_SUCCESS;
                return _result;
            }


            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD );
            return _result;
        }

        // OK
        if ( count )
        {			
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, _packetData, count, IV_PENDING_FINGERPRINT_TRIALS( getTimeout() ) )) != IV_ERROR_SUCCESS )
        {			
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK );
            return _result;
        }

        // Templatetype uebernehmen
        type = _packetData[0];
        // Templatedaten uebernehmen
        size = count - 1;
        // Templatedaten muessen mindestens 15 Byte haben.
        if ( size < 15 )
            return -IV_ERROR_PROCESSING;

        for ( i = 1; i < count; ++i )
        {		
            data[i - 1] = _packetData[i];
        }
        count += size;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::fingerprintTableGetRecordOk( short deviceID )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
    }
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::fingerprintTableDeleteRecord( short deviceID, unsigned int pid, unsigned char fid )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, pid, fid );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );

        // Parameter eintragen
        *reinterpret_cast<unsigned int *>( &_packetData[count] ) = pid;
        count = sizeof( unsigned int );
        _packetData[count] = fid;
        count += sizeof( unsigned char );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Wenn die Funktion unbekannt ist,
            // wird die Vorgaengerversion ohne "Aktive Antwort" geprueft.
            if ( ((_result >> 8) & 0xFF) == 2 )
            {
                // Vorgaengerfunktion ausfuehren.
                if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1, _packetData, count )) != IV_ERROR_SUCCESS )
                {
                    // Fehler ausgeben
                    traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1 );
                    return _result;
                }

                // OK
                if ( count )
                {
                    trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
                }

                // OK zurueckmelden
                _result = IV_ERROR_SUCCESS;
                return _result;
            }

            // Fehler ausgeben
            traceError( deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD );
            return _result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, _packetData, count, IV_PENDING_FINGERPRINT_TRIALS( getTimeout() ) )) != IV_ERROR_SUCCESS )
        {			
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK );
            return _result;
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::fingerprintTableDeleteRecordOk( short deviceID )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGenerate( short deviceID, char *name, char *reserved )
{
    trace( "%s(%i):INFO(3): %s [ %i, %s, %s ]\n", __FILE__, __LINE__, __func__, deviceID, name, reserved );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceID );

        // Alle Zusatzparameter uebernehmen
        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", name ) + 1;
        count += sprintf( reinterpret_cast<char *>( &_packetData[count] ), "%s", reserved ) + 1;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
        else
        {
            count = 0;
        }

        // Aktive Antwort abwarten
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, _packetData, count, IV_PENDING_FINGERPRINT_TRIALS( getTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK );
            return _result;
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleTable::tableGenerateOk( short deviceID )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceID )
    {
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

