/**
 * @file  FileTouchKeys.h
 * @brief Schnittstelle fuer die Klasse FileTouchKeys.
 */
//////////////////////////////////////////////////////////////////////
#ifndef FILETOUCHKEYS_H
#define FILETOUCHKEYS_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

// Verwaltungsklasse fuer Datei
#include "fileintelhex.h"
// 16bit/32bit CRC
#include "../Common/crc.h"

// Strukturdefinitionen
#include "../Common/pzemaiv.h"

// Maximale Anzahl Datenbyte (1kByte)
#define MAX_MEM_TOUCHKEYS   0x400

// Maximale Anzahl TouchKeys
#define MAX_TOUCHKEYS   100

class FileTouchKeys : public FileIntelHex
{
public:
	FileTouchKeys();
	virtual ~FileTouchKeys();

public:
	unsigned int m_sizeData;

	// Verwaltungsstrukturen fuer TouchKey
	TOUCHKEYNODE *m_pTouchKeysNode;
	TOUCHKEY     *m_pTouchKeys[MAX_TOUCHKEYS];

	// Speicher fuer alle Daten
	unsigned char m_memTouchKeys[MAX_MEM_TOUCHKEYS];

public:
	void reset();
	int Load(const char *fileName);
	int Save(const char *fileName, unsigned char append = 0);
	inline unsigned int GetSizeData() { return m_sizeData; };
  void Init(unsigned char count, unsigned char deviceNumber);
	int SetTouchKey(int idx, TOUCHKEY tk);
	TOUCHKEY GetTouchKey(int idx);
	int GetTouchKeyCount()
	{
		return m_pTouchKeysNode->cnt;
	}

    int GetDeviceNumber()
    {
        return m_pTouchKeysNode->temp[0];
    }

protected:
	int SaveTouchKeys(const char *fileName, unsigned char append = 0);
};

#endif

